/*
 * Momentum Risk & Savings
 * Created on 2005/09/23
 *
 */
package za.co.momentum.recurring.base.email;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
//import javax.activation.DataHandler;
//import javax.activation.FileDataSource;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import org.apache.log4j.Logger;
import za.co.momentum.recurring.base.Util;
import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import za.co.momentum.recurring.nurse.client.sao.appreq.AppReqServiceManager;
import za.co.momentum.recurring.nurse.util.ClientProperties;

/**
 * Implementation of an email client
 * 
 * @author Deon van Aarde
 *  
 */
public class EmailClient {
  private static final Logger LOG = Logger.getLogger(EmailClient.class);
  private final Session session;
  private final ClientProperties properties;
  private static EmailClient instance;

  public static EmailClient getInstance() {
  	if (instance == null) {
  		synchronized (EmailClient.class) {
  			instance = new EmailClient();
			}
  	}
  	return instance;
  }
  
  private EmailClient() {
    properties = ClientProperties.getInstance();
    Properties props = new Properties();
//    props.put("mail.pop3.host", properties.getProperty("email.popServer"));
//    props.put("mail.smtp.host", properties.getProperty("email.smtpServer"));
    session = Session.getDefaultInstance(props, null);
  }

  public void sendTechnicalAlert(String subject, String message) {
    String fromAddr = properties.getProperty("email.fromAddress");
    String toAddr = properties.getProperty("email.techicalAddress");
    sendMail(fromAddr, toAddr, subject, message, null);
  }

  public void sendTechnicalAlert(String subject, String message, String filename) {
    String fromAddr = properties.getProperty("email.fromAddress");
    String toAddr = properties.getProperty("email.techicalAddress");
    Collection col = new ArrayList();
    col.add(filename);
    sendMail(fromAddr, toAddr, subject, message, col);
  }

  public void sendTechnicalAlert(String username, String function,
      Exception error) {
    String fromAddr = properties.getProperty("email.fromAddress");
    String toAddr = properties.getProperty("email.techicalAddress");
    sendMail(fromAddr, toAddr, username + " " + function, error.toString(),
        null);
  }

  public void sendMail(String fromAddress, String toAddress, String subject,
      String bodyText, Collection fileNames) {
    if (Util.isEmpty(fromAddress) || fromAddress.length() < 8)
      throw new IllegalArgumentException("Invalid fromAddress: " + fromAddress);
    if (Util.isEmpty(toAddress) || toAddress.length() < 8)
      throw new IllegalArgumentException("Invalid toAddress: " + toAddress);
    if (Util.isEmpty(bodyText))
      throw new IllegalArgumentException("Invalid bodyText: " + bodyText);
    long startTime = System.currentTimeMillis();
    LOG.debug("Sending email from " + fromAddress + " to " + toAddress
        + " subject: " + subject);
    
    try{
    	AppReqServiceManager.getInstance().sendGenericEmail(fromAddress, toAddress, subject, bodyText, fileNames);
    	LOG.debug("Email send in " + Util.getTimelapse(startTime) + "ms.");
    } catch (Exception e) {
    	LOG.debug("Email not send in " + Util.getTimelapse(startTime) + "ms.");
    	throw new DomainRuntimeException(e);
	}
    
    /*try {
      MimeMessage message = new MimeMessage(session);
      message.setFrom(new InternetAddress(fromAddress));
      InternetAddress[] address = { new InternetAddress(toAddress) };
      message.setRecipients(Message.RecipientType.TO, address);
      message.setSubject(subject);
      message.setSentDate(new Date());
      MimeBodyPart mimeBody = new MimeBodyPart();
      mimeBody.setText(bodyText);
      Multipart multiPart = new MimeMultipart();
      multiPart.addBodyPart(mimeBody);
      if (!Util.isEmpty(fileNames)) {
        LOG.debug("Found " + fileNames.size() + " file attachments");
        Iterator files = fileNames.iterator();
        while (files.hasNext()) {
          String file = (String) files.next();
          MimeBodyPart fileMimeBody = new MimeBodyPart();
//          FileDataSource fds = new FileDataSource(file);
//          fileMimeBody.setDataHandler(new DataHandler(fds));
//          fileMimeBody.setFileName(fds.getName());
//          multiPart.addBodyPart(fileMimeBody);
        }
      }
      message.setContent(multiPart);
      Transport.send(message);
    } catch (AddressException ae) {
      LOG.error(ae);
      throw new DomainRuntimeException(ae);
    } catch (MessagingException me) {
      LOG.error(me);
      throw new DomainRuntimeException(me);
    }*/ finally {
      LOG.debug("Email send in " + Util.getTimelapse(startTime) + "ms.");
    }
  }
}
