
/*
 * Momentum Risk & Savings
 * Created on 2005/08/24
 *
 */
package za.co.momentum.recurring.base.ftp;


import org.apache.log4j.Logger;

import za.co.momentum.recurring.base.Util;
import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import ftp.FtpBean;
import ftp.FtpObserver;

/**
 * It connects to a ftp server.
 * 
 * Note that this class implements the FtpObserver interface, which make this
 * class have the ability to monitor the downloading or uploading progress. If
 * you don't need to monitor it, then you don't need to implement this
 * interface.
 * 
 * For using more function of this FtpBean, please see the documentation.
 */
public class FtpClient implements FtpObserver  {
	
	private static final Logger LOG = Logger.getLogger(FtpClient.class);
	private FtpBean ftp;
	private long bytes = 0;

	public FtpClient(){}
	
	/**
	 * Connects to a FTP site 
	 * @param ftpSite
	 * @param username
	 * @param password
	 */	
	public void connect(String ftpSite, String username, String password) {
	  long start = System.currentTimeMillis();
	  try {
		  LOG.debug("Connecting to " + ftpSite + " as " + username);
	  	ftp = new FtpBean();
	  	ftp.setPassiveModeTransfer(false); // safer
	  	ftp.ftpConnect(ftpSite, username, password);
		} catch (Exception e) {
			throw new DomainRuntimeException("Unable to connect " + ftpSite, e);
		} finally {
		  LOG.debug("Connect took " + Util.getTimelapse(start) + "ms");
		}
		
	}

	
	/**
	 * Close FTP connection
	 *
	 */
	public void close() {
		try {
			ftp.close();
		} catch (Exception e) {
		  throw new DomainRuntimeException("Unable to close connection", e);
		}
	}

	/**
	 * Copy binary file from FTP site 
	 * @param ftpFilename
	 * @param localFilename
	 */
	public void getFile(String ftpFilename, String localFilename) {
	  long start = System.currentTimeMillis();
		try {
		  LOG.debug("Getting FTP file " + ftpFilename + " to " + localFilename);
		  bytes = 0;
			ftp.getBinaryFile(ftpFilename, localFilename, this);
		} catch (Exception e) {
		  throw new DomainRuntimeException("Unable to get ftp file " + ftpFilename, e);
		} finally {
		  LOG.debug("Copying of " + bytes + " bytes took " + Util.getTimelapse(start) + "ms");
		}
	}
	
	/**
	 * Copy binary file to FTP site
	 * @param localFilename
	 * @param ftpFilename
	 */
	public void putFile(String localFilename, String ftpFilename) {
	  long start = System.currentTimeMillis();
		try {
		  LOG.debug("Putting " + localFilename + " to FTP file " + ftpFilename);
		  bytes = 0;
			ftp.putBinaryFile(ftpFilename, localFilename, this);
		} catch (Exception e) {
		  throw new DomainRuntimeException("Unable to put file " + localFilename, e);
		} finally {
		  LOG.debug("Copying of " + bytes + " bytes took " + Util.getTimelapse(start) + "ms");
		}
	}

	/**
	 * FtpObserver
	 */
	public void byteRead(int bytes) {
		this.bytes += bytes;
//		LOG.debug(this.bytes + " of bytes read already");
	}

	/**
	 * FtpObserver 	
	 */
	public void byteWrite(int bytes) {
	  this.bytes += bytes;
//		LOG.debug(this.bytes + " of bytes written already");
	}

}
