package za.co.momentum.recurring.client.TNPlugin;

import java.io.File;
import java.io.FileWriter;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.eclipse.core.runtime.IPlatformRunnable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.preferences.ConfigurationScope;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPreferenceConstants;
import org.eclipse.ui.PlatformUI;

import za.co.momentum.recurring.nurse.client.model.ClientSecurityManager;
import za.co.momentum.recurring.nurse.client.tools.PdfFileManager;
import za.co.momentum.recurring.nurse.client.ui.FastLanePreferencePage;
import za.co.momentum.recurring.nurse.client.ui.LoginWindow;
import za.co.momentum.recurring.nurse.client.update.UpdateAction;
import za.co.momentum.recurring.nurse.db.DBManager;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.FastlaneFormBackup;
import za.co.momentum.recurring.nurse.util.TNLogger;
import za.co.momentum.recurring.nurse.util.WebCaller;
import za.co.momentum.recurring.nurse.util.adobe.AdobeUtils;

import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * This class controls all aspects of the application's execution
 */
public class Application implements IPlatformRunnable {
  public static final String PLUGIN_ID = "TNPlugin";

  public static final String APPLICATION_NAME = "Fastlane MedPerson";

  private static final String EFORM_BACKUP_DIRECTORY = ClientProperties.getInstance().getProperty(
      ClientProperties.EFORM_BACKUP_DIRECTORY);

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.core.runtime.IPlatformRunnable#run(java.lang.Object)
   */
  public Object run(Object args) throws Exception {
    // URL fileURL =
    // AspireLoggingFrameworkPlugin.getDefault().getBundle().getEntry("log4j.properties");
    // PropertyConfigurator.configure("." + File.separator
    // + Platform.getProduct().getDefiningBundle().getLocation()
    // .substring(
    // Platform.getProduct().getDefiningBundle().getLocation().indexOf(
    // "@") + 1) + "log4j.properties");
    PropertyConfigurator.configure(Platform.getProduct().getDefiningBundle().getEntry("log4j.properties"));
    Logger.getLogger(Application.class).info("Starting up fastlane");
    
    if ("windows".equalsIgnoreCase(System.getProperty("os.name"))){
      AdobeUtils.detectAndSaveInstallPath();
    }else {
    	IEclipsePreferences preferences = ConfigurationScope.INSTANCE
				.getNode(Application.PLUGIN_ID);
    	//DK removed preferences.put(FastLanePreferencePage.PDF_EDITOR_PATH, "/usr/bin/evince");
    }



    File f = new File("client-config.wsdd");
    if (!f.exists()) {
      FileWriter out = new FileWriter("client-config.wsdd");
      out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?> " + "<deployment " + "name=\"commonsHTTPConfig\"  "
          + "xmlns=\"http://xml.apache.org/axis/wsdd/\"  "
          + "xmlns:java=\"http://xml.apache.org/axis/wsdd/providers/java\"> "
          + "<transport name=\"http\" pivot=\"java:org.apache.axis.transport.http.CommonsHTTPSender\" /> "
          + "<transport name=\"local\" pivot = \"java:org.apache.axis.transport.local.LocalSender\" /> "
          + "<transport name=\"java\" pivot=\"java:org.apache.axis.transport.java.JavaSender\" /> " + "</deployment> ");
      out.flush();
      out.close();
      Logger.getLogger(Application.class).info("Restarting application");
      return IPlatformRunnable.EXIT_RESTART;
    }
    // PreferenceInitializer iniPrefs = new PreferenceInitializer();
    // iniPrefs.initializeDefaultPreferences();
    // Logger.getLogger(Application.class).info("Preferences defaults loaded");
    Display display = PlatformUI.createDisplay();
    UpdateAction update = new UpdateAction();
    update.run();

    if (update.isExitApp()) {
      return IPlatformRunnable.EXIT_OK;
    }

    // Before we logon, we must first try to update any changed versions.
    if (!update.isRestart()) {
      // VersionServiceManager.getInstance().updateAll();
      // Add in here stuff that must be done before loggin on
      System.out.println("Starting app normally");

    } else {
      Logger.getLogger(Application.class).info("Restarting application");
      return IPlatformRunnable.EXIT_RESTART;
    }

    // Start db
    TNLogger.debug("Starting local database");
    DBManager.getinstance().startDB();

//    // Do plugin startup here
//    try {
//      Platform.getBundle("MedPersonClientWeb").start();
//
//    } catch (BundleException e) {
//      Logger.getLogger(ApplicationWorkbenchWindowAdvisor.class).fatal(
//          "Could not start client web app " + e.getMessage());
//      // SendErrorEmail.sendErrorMail(e, "postWindowOpen", "ApplicationWorkbenchAdvisor");
//    }

    // Start webcaller
    Thread t = new Thread(WebCaller.getIntance());
    t.start();
    PlatformUI.getPreferenceStore().setValue(IWorkbenchPreferenceConstants.SHOW_PROGRESS_ON_STARTUP, true);
    Platform.endSplash();

    // Create work and backup directories
    File backupLocationDir = new File(EFORM_BACKUP_DIRECTORY);
    if (!backupLocationDir.exists()) {
      String main = EFORM_BACKUP_DIRECTORY.substring(0, EFORM_BACKUP_DIRECTORY.lastIndexOf('/'));
      File mainDir = new File(main);
      if (mainDir.exists()) {
        backupLocationDir.mkdir();
      } else {
        if (mainDir.mkdir()) {
          backupLocationDir.mkdir();
        }
      }
    }
    File workDirectoy = new File(PdfFileManager.getWorkDirectory());
    if (!workDirectoy.exists()) {
      workDirectoy.mkdir();
    }

    // Clean out all backups older than 90 days
    try {
      FastlaneFormBackup formBackup = new FastlaneFormBackup();
      formBackup.removeBackupsOlderThanXDays(90);
    } catch (Exception e) {
      e.printStackTrace();
    }

    if (!update.isRestart()) {
      LoginWindow lw = new LoginWindow();
      lw.open();
    }
    if (ClientSecurityManager.getInstance().getUser() == null) {
      SWTResourceManager.dispose();
      ResourceManager.dispose();
      return IPlatformRunnable.EXIT_OK;
    } else {
      try {
        int returnCode = PlatformUI.createAndRunWorkbench(display, new ApplicationWorkbenchAdvisor());
        if (returnCode == PlatformUI.RETURN_RESTART) {
          return IPlatformRunnable.EXIT_RESTART;
        }
        return IPlatformRunnable.EXIT_OK;
      } finally {
        display.dispose();
        SWTResourceManager.dispose();
        ResourceManager.dispose();
      }
    }
  }
}
