package za.co.momentum.recurring.client.TNPlugin.actions;

// Test comment
import java.lang.reflect.Method;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;

import com.swtdesigner.SWTResourceManager;


import za.co.momentum.recurring.nurse.client.model.ClientSecurityManager;
import za.co.momentum.recurring.nurse.client.tools.SendErrorEmail;
import za.co.momentum.risk.fastlane.nurse.dto.RoleTO;
import za.co.momentum.risk.fastlane.nurse.dto.UserTO;


public abstract class ActionAdapter implements IWorkbenchWindowActionDelegate {

	private static final Logger LOG = Logger.getLogger(ActionAdapter.class);
	public void dispose() {
	}

	public void init(IWorkbenchWindow window) {
	}

	public void run(IAction action) {
		LOG.debug("ActionAdapter::run");
		try {
			try { 
				   PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().setCursor(SWTResourceManager
						.getCursor(SWT.CURSOR_WAIT));
				} catch (IllegalStateException e) {} // NOt a problem, no workbench yet.
			IWorkbenchPart view = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
					.getActivePart(); 
			UserTO user = ClientSecurityManager.getInstance().getUser();
			
			if (getActionRole() !=  -1) {
				if (!isInRole(getActionRole())) {
					LOG.debug("ACCESS DENIED: Not in role " + getActionRole());
					return;
				}
			}
			if (getActionRoles() != null) {
				if (!isInListOfRoles()) {
					LOG.debug("ACCESS DENIED: Not in list of roles");
					return;
				}
			}
			LOG.debug("Calling runManaged(action, view, user");
			runRunManaged(action, view, user);
		} catch (Exception e) {
			e.printStackTrace();
			SendErrorEmail.sendErrorMail(e, "ActionAdapter", "");
			throw new RuntimeException(e);
		} finally {
			try {
			    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().setCursor(SWTResourceManager
					.getCursor(SWT.CURSOR_ARROW));
			} catch (IllegalStateException e) {} // Not a problem.
		}
	}
	/*
	 * 
	 * This massive inderiction enables us to lose the cast to of IWorkbenchPart in the Action.
	 */
	private void runRunManaged(IAction action, IWorkbenchPart view, UserTO user) throws Exception {
		Class cl = this.getClass();
		
		Method mm = null;
		try {
			Method [] meths = cl.getMethods();
			for (int i = 0; i < meths.length; i++) {
				Method meth = meths[i];
				if ("runManaged".equals(meth.getName())) {
					
					mm = meth;
				}
			}
			
		} catch (Exception e) {
			LOG.debug("Inconsistent defintion for runManaged, most likely the wrong view active, not throwing an error." + e);
			return;
		}
		
		Object [] args = new Object [3];
		args[0] = action;
		args[1] = view;
		args[2] = user;
		try {
			mm.invoke(this, args);
		} catch (Exception e) {
			LOG.debug("Invoke failed on runManaged");
			throw e;
		}
		
	}
	// This definition has been lost in favour of a more flexible reflection approach.
	//public abstract void runManaged(IAction action, IWorkbenchPart view, UserTO user) throws Exception;

	public void selectionChanged(IAction action, ISelection selection) {
	}
	private boolean isInRole(int rid) {
		RoleTO [] roles = ClientSecurityManager.getInstance().getRoles();
		for (int i = 0; i < roles.length; i++) {
			RoleTO role = roles[i];
			if (role.getRoleType() == rid) return true;
		}
		return false;
	}
	private boolean isInListOfRoles() {
		int [] rids = getActionRoles();
		for (int i = 0; i < rids.length; i++) {
			if (isInRole(rids[i])) {
				return true;
			}
		}
		return false;
	}
	public int getActionRole() {
		return -1;
	}
	public int [] getActionRoles() {
		return null;
	}
	
}