package za.co.momentum.recurring.client.TNPlugin.actions;

import ie.wombat.jbdiff.JBUtil;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import za.co.momentum.recurring.base.Util;
import za.co.momentum.recurring.base.email.EmailClient;
import za.co.momentum.recurring.client.TNPlugin.Application;
import za.co.momentum.recurring.nurse.client.model.ClientSecurityManager;
import za.co.momentum.recurring.nurse.client.model.ClientWindowManager;
import za.co.momentum.recurring.nurse.client.sao.ServiceAction;
import za.co.momentum.recurring.nurse.client.tools.PdfFileManager;
import za.co.momentum.recurring.nurse.client.ui.NewNurseCalendarView;
import za.co.momentum.recurring.nurse.client.ui.ProgressDialog;
import za.co.momentum.recurring.nurse.client.ui.tools.ContinuousProgressBar;
import za.co.momentum.recurring.nurse.client.ui.tools.ProgressAction;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.WebCall;
import za.co.momentum.recurring.nurse.util.WebCaller;
import za.co.momentum.risk.fastlane.nurse.dto.AppClientTO;
import za.co.momentum.risk.fastlane.nurse.dto.AppDetailTO;
import za.co.momentum.risk.fastlane.nurse.dto.PdfFormTO;
import za.co.momentum.risk.fastlane.nurse.dto.RequirementTO;
import za.co.momentum.risk.fastlane.nurse.dto.UserTO;
import za.co.momentum.risk.fastlane.nurse.services.appreq.AppReqService;
import za.co.momentum.risk.fastlane.nurse.services.appreq.AppReqServiceRest;

import com.swtdesigner.SWTResourceManager;

public class AppointmentDoneAction extends ActionAdapter {
	PdfFormTO[] pdfFormTO;

	boolean doneAppSuccess;

	String[] failedForms;

	private static final Logger LOG = Logger
			.getLogger(AppointmentDoneAction.class);

	public int getActionRole() {
		return ClientSecurityManager.NURSE_ROLE;
	}

	public void runManaged(IAction action, NewNurseCalendarView nview,
			UserTO user, UserTO adminUser) throws Exception {

		AppDetailTO appReqDetailsTO = nview.getCurrentApp();
		if (appReqDetailsTO == null) {
			return;
		}

		// Get form statusses
		String formStatusMatrix = getFormSatusMatrix(appReqDetailsTO, user);

		// Check for blank or unsaved forms as well as missing forms and ask
		// user to continue
		if (!reviewForms(formStatusMatrix)) {
			return;
		}

		// NEW: Create backup of all forms currently in the client's folder
		AppClientTO[] appClientTOs = appReqDetailsTO.getClients();
		for (int i = 0; i < appClientTOs.length; i++) {
			String clientNumber = appClientTOs[i].getClientNumber();
			String clientName = appClientTOs[i].getPerson().getFirstName()
					+ " " + appClientTOs[i].getPerson().getSurname();
			PdfFileManager
					.backupAllFilesForClientNumber(
							appClientTOs[i].getPolicyAlpha(),
							appClientTOs[i].getPolicyNumber(), clientName,
							clientNumber);
		}

		// Now get all the diffs from all forms for this appreq
		List allPDFFormsList = new ArrayList();
		for (int i = 0; i < appReqDetailsTO.getClients().length; i++) {

			PdfFormTO[] pdfFormTOs = createPdfFormTOs(appReqDetailsTO
					.getClients()[i]);

			for (int j = 0; j < pdfFormTOs.length; j++) {
				allPDFFormsList.add(pdfFormTOs[j]);
			}
		}

		// Create unique combinations of client numbers and form names
		Map uniquePdfFormsMap = new TreeMap();
		for (Iterator iter = allPDFFormsList.iterator(); iter.hasNext();) {
			PdfFormTO pdfFormTO = (PdfFormTO) iter.next();
			String formIdentifier = pdfFormTO.getClientNumber() + "_"
					+ pdfFormTO.getFormName();
			uniquePdfFormsMap.put(formIdentifier, pdfFormTO);
		}

		// Create array of all the PDFFormTO's
		PdfFormTO[] pdfForms = new PdfFormTO[uniquePdfFormsMap.size()];
		Iterator it = uniquePdfFormsMap.entrySet().iterator();
		int k = 0;
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			PdfFormTO pdfForm = (PdfFormTO) entry.getValue();
			pdfForms[k] = pdfForm;
			k++;
		}

		try {
			// First try to submit all forms to server and check for any failed
			// forms
			failedForms = submitAppDoneToServer(user, adminUser,
					appReqDetailsTO, pdfForms);

			// TODO: Removed deleting of files in work folder so that MP's can
			// always get to forms
			// Now move all successful files to the sent directory
			// removeAllSuccessfullFromWorkDir(appReqDetailsTO, pdfForms);

			if (doneAppSuccess) {
				displayAppDoneSuccess(appReqDetailsTO);

			} else {
				// Forms have failed, try sending through the whole form, and
				// not just the diff.
				if (failedForms != null && failedForms.length > 0) {
					PdfFormTO[] failedFormsFull = populateFailedPDFsWithFullBytes(pdfForms);

					failedForms = submitAppDoneToServer(user, adminUser,
							appReqDetailsTO, failedFormsFull);

					// Now move all successful files to the sent directory (DO
					// NOT DELETE FORMS ANYMORE!)
					// removeAllSuccessfullFromWorkDir(appReqDetailsTO,
					// pdfForms);

					// If forms still failed, send warning to MedPerson to
					// contact support
					if (failedForms != null && failedForms.length > 0) {
						displayFailedFormsWarning();
					} else {
						displayAppDoneSuccess(appReqDetailsTO);
					}
				} else {
					// If no forms have failed, it might be a connectivity or
					// other issue, so submit to background queue
					submitAppDoneToBackgroundCaller(user, adminUser,
							appReqDetailsTO, pdfForms);
				}
			}

		} catch (Exception e) {
			LOG.error(e);
		}

		// Now reload the calendar
		nview.reloadDay();
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell()
				.setCursor(SWTResourceManager.getCursor(SWT.CURSOR_ARROW));
	}

	private String[] submitAppDoneToServer(final UserTO userTO,
			final UserTO adminUser, final AppDetailTO appDetailTO,
			final PdfFormTO[] pdfFormTOs) {

		ContinuousProgressBar prog = new ContinuousProgressBar(
				PlatformUI.isWorkbenchRunning() ? PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell() : new Shell(),
				SWT.NULL, new ProgressAction() {
					public void executeAction() {

						// AppReqServiceServiceLocator loc = new
						// AppReqServiceServiceLocator();
						try {
							// AppReqService serv = loc.getAppReqService();

							AppReqService service = AppReqServiceRest
									.getInstance();

							failedForms = service.setAppointmentDoneAction(
									userTO, adminUser, appDetailTO, pdfFormTOs);

							if (failedForms == null || failedForms.length == 0) {
								doneAppSuccess = true;
							} else {
								doneAppSuccess = false;
							}
						} catch (Exception e) {
							doneAppSuccess = false;
						}
					}
				});
		prog.setHeadingDescription(Application.APPLICATION_NAME);
		prog.setActionDescription("Submitting completed forms...");
		try {
			prog.open();

		} catch (Exception e) {
			e.printStackTrace();

			doneAppSuccess = false;
		}

		return failedForms;
	}

	private void submitAppDoneToBackgroundCaller(final UserTO userTO,
			final UserTO adminUser, final AppDetailTO appDetailTO,
			final PdfFormTO[] pdfFormTOs) {

		String messageText = "A server error (connectivity or otherwise) occurred, "
				+ "the form(s) will now be submitted to the background queue.";
		MessageBox message = new MessageBox(ClientWindowManager.getInstance()
				.getShell(), SWT.OK + SWT.ICON_ERROR);
		message.setMessage(messageText);
		message.setText("Connectivity Problem");
		message.open();

		WebCall wc = new WebCall(new ServiceAction() {
			public Object performService() throws Exception {
				// AppReqServiceServiceLocator loc = new
				// AppReqServiceServiceLocator();
				// // loc.setCacheWSDL(false);
				// AppReqService serv = loc.getAppReqService();

				AppReqService service = AppReqServiceRest.getInstance();

				return service.setAppointmentDoneAction(userTO, adminUser,
						appDetailTO, pdfFormTOs);
			}
		}, appDetailTO.getId(), 0, "Appointment done "
				+ appDetailTO.getPolicyNoAlpha() + appDetailTO.getPolicyNo()
				+ " " + appDetailTO.getContactPerson().getFirstName() + " "
				+ appDetailTO.getContactPerson().getSurname());
		WebCaller.getIntance().addWebCall(wc);
	}

	private void displayAppDoneSuccess(AppDetailTO appDetailTO) {
		String doneMessage = "Appointment done: "
				+ appDetailTO.getPolicyNoAlpha().trim()
				+ appDetailTO.getPolicyNo().trim() + " - "
				+ appDetailTO.getContactPerson().getFirstName().trim() + " "
				+ appDetailTO.getContactPerson().getSurname().trim();

		ProgressDialog progDialog = new ProgressDialog(ClientWindowManager
				.getInstance().getShell(), SWT.NULL);
		progDialog.setMessage(doneMessage);
		progDialog.open();
	}

	private boolean reviewForms(String formStatusMatrix) {

		if (formStatusMatrix.indexOf("Blank (or unsaved) form") > -1
				|| formStatusMatrix.indexOf("Form not found") > -1) {
			MessageBox messageBox = new MessageBox(ClientWindowManager
					.getInstance().getShell(), SWT.ICON_WARNING | SWT.YES
					| SWT.NO);
			messageBox.setText("Incomplete Appointment Detected");
			String message = "An incomplete appointment was detected. This might be due to a blank or unsaved form or "
					+ "a required form not found in this client's work folder.\n\nPlease review all forms and their statusses below "
					+ "before continuing.\n\n";
			message += formStatusMatrix;
			message += "* Note: Blank forms will not be sent to the server in order to save bandwidth and space.\n\n"
					+ "Do you want to continue with this Appointment Done action?";
			messageBox.setMessage(message);

			int selectionId = messageBox.open();
			if (selectionId == SWT.NO) {
				return false;
			}
		}
		return true;
	}

	private String getFormSatusMatrix(AppDetailTO appReqDetailsTO, UserTO user) {

		Map formStatusses = new TreeMap();

		// Get all the clients on this app
		AppClientTO[] clients = appReqDetailsTO.getClients();

		// Loop through all the clients
		for (int i = 0; i < clients.length; i++) {
			// Get all requirements for this client
			RequirementTO[] reqs = clients[i].getRequirements();

			// Put form number for this requirement into map
			for (int j = 0; j < reqs.length; j++) {
				if (reqs[j].getGroup() != null
						&& reqs[j].getGroup().equals("Additional")
						&& !(user.getId() + "").equals(reqs[j].getMedPerson())) {
					continue;
				}

				String reqFormName = clients[i].getClientNumber()
						+ "_" + getFormNo(reqs[j].getFormNo())
						+ clients[i].getPerson().getLanguage() + ".pdf";

				// Skip ECG form
				if (reqFormName.length() == 0
						|| reqFormName.indexOf("ECG") >= 0) {
					continue;
				}

				String formStatus = PdfFileManager.getFileStatus(
						clients[i].getClientNumber(), getFormNo(reqFormName),
						clients[i].getPerson().getLanguage());

				formStatusses.put(reqFormName, formStatus);
			}
		}

		// Get statusses of completed forms and create string for display
		// purposes
		String formStatusMatrix = "";
		int m = 0;
		Iterator it = formStatusses.entrySet().iterator();
		while (it.hasNext()) {
			m++;
			Map.Entry entry = (Map.Entry) it.next();
			String formName = (String) entry.getKey();
			String formStatus = (String) entry.getValue();

			formStatusMatrix += "  " + m + ". " + formName + " - " + formStatus
					+ "\n\n";
		}

		return formStatusMatrix;
	}

	private void removeAllSuccessfullFromWorkDir(AppDetailTO appReqDetailsTO,
			PdfFormTO[] pdfForms) {

		// Get all successfull files from total List
		List allFormNames = new ArrayList();
		for (int i = 0; i < pdfForms.length; i++) {
			allFormNames.add(pdfForms[i].getClientNumber() + "_"
					+ pdfForms[i].getFormName());
		}

		if (failedForms != null) {
			if (failedForms.length > 0) {
				List failedFormsNames = new ArrayList();
				for (int i = 0; i < failedForms.length; i++) {
					failedFormsNames.add(failedForms[i]);
				}
				// Now remove all names of failed forms so only successfull form
				// names are left
				allFormNames.removeAll(failedFormsNames);
			}
		}

		// Now move all successfull pdf forms to the sent folder
		// for (Iterator iter = allFormNames.iterator(); iter.hasNext();) {
		// String formIdentifier = (String) iter.next();
		// String clientNumber =
		// getClientNumberFromFormIdentifier(formIdentifier);
		// String formName = getFormNameFromFormIdentifier(formIdentifier);
		//
		// copyPdfToSentDirectory(clientNumber, formName);
		// }

		// Now delete all successfull pdf forms from the work folder
		for (Iterator iter = allFormNames.iterator(); iter.hasNext();) {
			String formIdentifier = (String) iter.next();
			String clientNumber = getClientNumberFromFormIdentifier(formIdentifier);
			String formName = getFormNameFromFormIdentifier(formIdentifier);

			removePdfFromWorkDirectory(clientNumber, formName);
		}
	}

	private String getFormNameFromFormIdentifier(String formIdentifier) {
		int _place = formIdentifier.indexOf("_");
		return formIdentifier.substring(_place + 1, formIdentifier.length());

	}

	private String getClientNumberFromFormIdentifier(String formIdentifier) {
		int _place = formIdentifier.indexOf("_");
		return formIdentifier.substring(0, _place);
	}

	private PdfFormTO[] populateFailedPDFsWithFullBytes(PdfFormTO[] pdfForms) {

		// Get all failed forms PDFFormTO objects from original array (only use
		// uniques)
		Map failedPdfFormsMap = new TreeMap();

		for (int i = 0; i < pdfForms.length; i++) {
			for (int j = 0; j < failedForms.length; j++) {
				String formIdentifier = pdfForms[i].getClientNumber() + "_"
						+ pdfForms[i].getFormName();
				if (formIdentifier.equalsIgnoreCase(failedForms[j])) {
					failedPdfFormsMap.put(formIdentifier, pdfForms[i]);
				}
			}
		}

		PdfFormTO[] failedPdfFormTOs = new PdfFormTO[failedPdfFormsMap.size()];
		int i = 0;

		List failedPdfFormsList = new ArrayList(failedPdfFormsMap.values());

		for (Iterator iter = failedPdfFormsList.iterator(); iter.hasNext();) {
			PdfFormTO pdfFormTO = (PdfFormTO) iter.next();
			pdfFormTO.setPatch(false);
			pdfFormTO.setCheckSum(0);
			pdfFormTO.setZipFileBytes(getCompressedBytesFromPdf(pdfFormTO));

			failedPdfFormTOs[i] = pdfFormTO;
			i++;
		}

		return failedPdfFormTOs;
	}

	private void displayFailedFormsWarning() {

		// Display msg to MedPerson that some forms have failed
		String title = "Error generating forms on server";
		StringBuffer msg = new StringBuffer();

		msg.append("One or more forms could not be successfully saved on the Fastlane server side for\n"
				+ "this appointment.\n\n"
				+ "Please contact Fastlane Support and state the policy number, client number\nand form number(s) "
				+ "that failed:\n\n");

		for (int i = 0; i < failedForms.length; i++) {
			msg.append(" - " + failedForms[i] + "\n");
		}

		MessageBox message = new MessageBox(ClientWindowManager.getInstance()
				.getShell(), SWT.OK + SWT.ICON_ERROR);
		message.setMessage(msg.toString());
		message.setText(title);
		message.open();

	}

	private byte[] getCompressedBytesFromPdf(PdfFormTO pdfFormTO) {

		try {
			File f = PdfFileManager.getFileFromClientDetailsNotZipped(pdfFormTO
					.getClientNumber(), getFormNo(pdfFormTO.getFormName()),
					pdfFormTO.getLang().startsWith("A") ? "A" : "E");

			// Now zip the bytes to preserve space
			return Util.compressBytes(PdfFileManager.getByteArrayForFile(f));

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Get all the selected pdf forms to send, create binary diffs and load
	 * ready for transportation
	 * 
	 * @param clientNo
	 * @param language
	 * @throws IOException
	 */
	private PdfFormTO[] createPdfFormTOs(AppClientTO client) throws IOException {

		ArrayList pdfForms = new ArrayList();

		// Create map of unique forms (multiple reqs can have 1 form)
		Map uniqueReqs = new HashMap();
		for (int i = 0; i < client.getRequirements().length; i++) {
			uniqueReqs.put(client.getRequirements()[i].getFormNo(),
					client.getRequirements()[i]);
		}

		// Loop through unique requirements
		Iterator it = uniqueReqs.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			RequirementTO requirementTO = (RequirementTO) entry.getValue();

			// Check if the form for this requirement is blank, then do not send
			// to server
			String formStatus = PdfFileManager.getFileStatus(
					client.getClientNumber(),
					getFormNo(requirementTO.getFormNo()), client.getPerson()
							.getLanguage().startsWith("A") ? "A" : "E");

			if (formStatus.equalsIgnoreCase("Blank (or unsaved) form")
					|| formStatus
							.equalsIgnoreCase("Form not found in work folder")) {
				continue;
			}

			// Get the relevant completed Pdf from the work directory
			File completedPdfFile = PdfFileManager
					.getFileFromClientDetailsNotZipped(
							client.getClientNumber(), getFormNo(requirementTO
									.getFormNo()), client.getPerson()
									.getLanguage().startsWith("A") ? "A" : "E");

			// If the file is not available, continue with next requirement
			if (completedPdfFile == null || !completedPdfFile.exists()) {
				continue;
			}

			PdfFormTO pdfTO = new PdfFormTO();
			pdfTO.setFormName(requirementTO.getFormNo());
			pdfTO.setCode(requirementTO.getCode());
			pdfTO.setLang(client.getPerson().getLanguage().startsWith("A") ? "A"
					: "E");
			pdfTO.setClientName(client.getPerson().getFirstName() + " "
					+ client.getPerson().getSurname());
			pdfTO.setRequirement(requirementTO);
			pdfTO.setClientNumber(client.getClientNumber());

			// Get the checksum of the completed Pdf form and add to TO
			long checksum = JBUtil.getChecksum(
					completedPdfFile.getAbsoluteFile()).longValue();
			pdfTO.setCheckSum(checksum);

			// Get the path to the blank template form
			String templateFilePath = PdfFileManager.getBlankPdfTemplateFile(
					getFormNo(requirementTO.getFormNo()), client.getPerson()
							.getLanguage().startsWith("A") ? "A" : "E");

			// Get the flattened file diff between the completed and template
			// forms
			// byte[] diffBytes =
			// PdfFileManager.getDiffFromFlattenedPdfFiles(completedPdfFile.getAbsolutePath(),
			// templateFilePath);

			// Get the normal file diff between the completed and template forms
			byte[] diffBytes = PdfFileManager.getDiffFromPdfFiles(
					completedPdfFile.getAbsolutePath(), templateFilePath);

			// Create diff file for testing
			// File diffFile = new File(completedPdfFile.getAbsolutePath() +
			// ".diff");
			// PdfFileManager.createFileFromBytes(diffFile, diffBytes);

			if (diffBytes != null) {
				pdfTO.setPatch(true);
				pdfTO.setZipFileBytes(diffBytes);

			} else {
				// The diff process has failed, send through whole form
				pdfTO.setPatch(false);
				byte[] compressedFullFileBytes = Util
						.compressBytes(PdfFileManager
								.getByteArrayForFile(completedPdfFile));
				pdfTO.setZipFileBytes(compressedFullFileBytes);
			}

			// Add this form to the outgoing arraylist
			pdfForms.add(pdfTO);

			// If a ECG Form (002) is present, ask medperson to add ECG results
			// file for ECG form
			if (requirementTO.getFormNo().equalsIgnoreCase("UNDERW002")) {

				File ecgFile = getEcgPdfForm();

				if (ecgFile != null) {
					byte[] ecgBytes = PdfFileManager.getEcgBytes(ecgFile);
					LOG.debug("Before byte array size: " + ecgBytes.length);

					// Now zip the bytes to preserve space
					byte[] compressedBytes = Util.compressBytes(ecgBytes);
					LOG.debug("After compression byte array size: "
							+ compressedBytes.length);

					PdfFormTO ecgPdfTO = new PdfFormTO();

					// Now populate PdfFormTO with relevant details
					ecgPdfTO.setClientName(client.getPerson().getFirstName()
							+ " " + client.getPerson().getSurname());
					ecgPdfTO.setClientNumber(client.getClientNumber());
					ecgPdfTO.setCode("ECG");
					ecgPdfTO.setFormName("ECG");
					ecgPdfTO.setLang(client.getPerson().getLanguage()
							.startsWith("A") ? "A" : "E");
					ecgPdfTO.setPatch(false);
					ecgPdfTO.setZipFileBytes(compressedBytes);

					// Populate RequirementTO with relevant details
					RequirementTO reqTO = new RequirementTO();
					reqTO.setClient(client);
					reqTO.setCode("ECG");
					reqTO.setDate(Calendar.getInstance());
					reqTO.setDescription("ECG results file");
					reqTO.setDoneTS(Calendar.getInstance().getTimeInMillis());
					reqTO.setFormNo("ECG");

					ecgPdfTO.setRequirement(reqTO);

					// Add the ecg to the pdfForms collection
					pdfForms.add(ecgPdfTO);
				}
			}
		}

		// Create outgoing array of PdfFormTOs arraylist
		PdfFormTO[] pdfFormTOs = null;
		pdfFormTOs = new PdfFormTO[pdfForms.size()];
		for (int i = 0; i < pdfForms.size(); i++) {
			pdfFormTOs[i] = (PdfFormTO) pdfForms.get(i);
		}

		// PdfFileManager.deleteZipFiles(client.getClientNumber()); - No more
		// .zip files used

		return pdfFormTOs;
	}

	private String getFormNo(String formName) {
		return PdfFileManager.getFormNo(formName);
	}

	/**
	 * When the pdfs has been sent the files are copied to the SENT directory
	 * using this method
	 * 
	 */
	// public void copyPdfsToSent(AppClientTO client) {
	// RequirementTO[] reqs = client.getRequirements();
	// for (int i = 0; i < reqs.length; i++) {
	// PdfFileManager.copyFileToSent(client.getClientNumber(),
	// getFormNo(reqs[i].getFormNo()), client.getPerson()
	// .getLanguage().startsWith("A") ? "A" : "E");
	// }
	// }
	/**
	 * When the pdfs have been sent the files are copied to the SENT directory
	 * using this method
	 * 
	 */
	// private void copyPdfToSentDirectory(String clientNumber, String formName)
	// {
	// PdfFileManager.copyFileToSent(clientNumber, getFormNo(formName));
	// }
	private void removePdfFromWorkDirectory(String clientNumber, String formName) {
		PdfFileManager.removeFileFromWork(clientNumber, getFormNo(formName));
	}

	private void sendMailToFastlaneBA(String subject, String body) {
		EmailClient.getInstance()
				.sendMail(
						ClientProperties.getInstance().getProperty(
								"email.fromAddress"),
						ClientProperties.getInstance().getProperty(
								"email.baAddress"), subject, body, null);
	}

	private void sendMailToTechnical(String subject, String body) {
		EmailClient.getInstance()
				.sendMail(
						ClientProperties.getInstance().getProperty(
								"email.fromAddress"),
						ClientProperties.getInstance().getProperty(
								"email.techicalAddress"), subject, body, null);
	}

	private File browseForEcg() {

		FileDialog fileChooser = new FileDialog(ClientWindowManager
				.getInstance().getShell(), SWT.OPEN);

		fileChooser.setText("Please select the ECG file for this client");
		fileChooser.setFilterPath(PdfFileManager.getAbsoluteInstallPath()
				+ PdfFileManager.PDF_SOURCE_DIRECTORY
				+ PdfFileManager.ECG_DIRECTORY);
		fileChooser.setFilterExtensions(new String[] { "*.pdf" });
		fileChooser.setFilterNames(new String[] { "PDF Files" + " (pdf)" });
		String fileName = fileChooser.open();
		if (fileName == null) {
			return null;
		}

		return new File(fileName);
	}

	private File getEcgPdfForm() {

		File ecgFile = null;

		// Disply info message to ask user to browse to ECG file to attach
		MessageBox messageBox = new MessageBox(ClientWindowManager
				.getInstance().getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
		messageBox.setText("ECG Requirement Found For Policy");
		String message = "Would you like to attach the ECG results (pdf format only) to the\n"
				+ "current appointment so that it is automatically sent to head office?\n\n"
				+ "Please note that large files could take some time to submit successfully.";

		messageBox.setMessage(message);

		int selectionId = messageBox.open();
		if (selectionId == SWT.YES) {

			boolean browseForFile = true;
			while (browseForFile) {
				browseForFile = false;
				ecgFile = browseForEcg();

				// Add the ECG to the PDFForm collection
				if (ecgFile != null) {
					// Disply info message to ask user to browse to ECG file to
					// attach
					MessageBox fileBox = new MessageBox(ClientWindowManager
							.getInstance().getShell(), SWT.ICON_QUESTION
							| SWT.YES | SWT.NO | SWT.CANCEL);
					fileBox.setText("ECG File Selected");
					String fileMessage = "The following ECG file was selected:\n\n"
							+ ecgFile.getAbsolutePath()
							+ "\n\n"
							+ "Please confirm that this is the correct file.";

					fileBox.setMessage(fileMessage);

					int fileSelectionId = fileBox.open();

					if (fileSelectionId == SWT.NO)
						browseForFile = true;

					if (fileSelectionId == SWT.CANCEL)
						ecgFile = null;
				}
			}
		}
		return ecgFile;
	}

	private String getFormName(String selection) {
		String subs = "";
		selection = selection.trim();
		try {
			subs = selection.substring(selection.indexOf("UNDERW"),
					selection.indexOf("UNDERW") + 9);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return subs;
	}
}
