package za.co.momentum.recurring.client.TNPlugin.actions;

import java.util.ArrayList;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;

import za.co.momentum.recurring.nurse.client.tools.PdfFileManager;
import za.co.momentum.recurring.nurse.client.ui.LanguageChooser;
import za.co.momentum.recurring.nurse.client.ui.PdfFileChooserDialog;

/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class BlankFormsAction implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public BlankFormsAction() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
		
    // Open the PDF File chooser dialog box, save a blank form to the appropriate folder and open
    // the form with a PDF editor
    LanguageChooser lang = new LanguageChooser(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
        .getShell(), SWT.NULL);
    lang.open();
    
    String language = lang.getLanguageSelected();    
    
    if (language == null) {
      return;
    }
    
    ArrayList pdfFiles = new ArrayList();
    pdfFiles = PdfFileManager.getAllPdfFileNameInDir(language);
    PdfFileChooserDialog fileChooser = new PdfFileChooserDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
        .getShell(), pdfFiles);
    int returnCode = fileChooser.open();

    if (returnCode == IDialogConstants.OK_ID) {
      String formName = fileChooser.getSelectedFile();
      String formNumber = PdfFileManager.getFormNo(formName);
      PdfFileManager.openBlankPdf(formNumber, "ClientName_UNDERW" + formNumber, language);      
    }
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
}