package za.co.momentum.recurring.nurse.client.model;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

public class ClientWindowManager {
	private static final Logger LOG = Logger.getLogger(ClientWindowManager.class);
	private Shell window;
	private MenuItem[] menus;
	private IStatusLineManager statusLine;
	private static ClientWindowManager instance;
	private boolean searchRequests;
	private boolean searchHistory;
	private boolean searchDone;

	public Shell getShell(){
		return window;
	}
	
	public boolean isSearchDone() {
		return searchDone;
	}

	public void setSearchDone(boolean searchDone) {
		this.searchDone = searchDone;
	}

	public boolean isSearchHistory() {
		return searchHistory;
	}

	public void setSearchHistory(boolean searchHistory) {
		this.searchHistory = searchHistory;
	}

	public boolean isSearchRequests() {
		return searchRequests;
	}

	public void setSearchRequests(boolean searchRequests) {
		this.searchRequests = searchRequests;
	}

	public static ClientWindowManager getInstance() {
		if (instance == null) {
			synchronized (ClientWindowManager.class) {
				if (instance == null) {
					instance = new ClientWindowManager();
				}
			}
		}
		return instance;
	}

	private ClientWindowManager() {
		window = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		menus = window.getMenuBar().getItems();
		searchRequests = false;
		searchHistory = false;
	}

	public void setMenus() {
		setNoMenus();
		if (ClientSecurityManager.getInstance().isAdminRole()) {
			setSystemAdministratorMenus();
		}
		if (ClientSecurityManager.getInstance().isNurseRole()) {
			setNurseMenus();
		}
	}

	private int getFileMenu() {
		for (int i = 0; i < menus.length; i++) {
			if (menus[i].getText().equalsIgnoreCase("File")) {
				return i;
			}
		}
		return 0;
	}

	private int getActionMenu() {
		for (int i = 0; i < menus.length; i++) {
			if (menus[i].getText().equalsIgnoreCase("Action")) {
				return i;
			}
		}
		return 0;
	}

	private int getViewMenu() {
		for (int i = 0; i < menus.length; i++) {
			if (menus[i].getText().equalsIgnoreCase("View")) {
				return i;
			}
		}
		return 0;
	}

	private int getAdministratorMenu() {
		for (int i = 0; i < menus.length; i++) {
			if (menus[i].getText().equalsIgnoreCase("Administrator")) {
				return i;
			}
		}
		return 0;
	}

	private MenuItem getSubMenu(int mainMenuIndex, String menuName) {
		Menu m = menus[mainMenuIndex].getMenu();
		if (m != null) {
			MenuItem[] mi = m.getItems();
			for (int i = 0; i < mi.length; i++) {
				if (mi[i].getText().equalsIgnoreCase(menuName)) {
					return mi[i];
				}
			}
		}
		return null;
	}

	private void disableSubMenu(int mainMenuIndex) {
		Menu m = menus[mainMenuIndex].getMenu();
		if (m != null) {
			MenuItem[] mi = m.getItems();
			for (int i = 0; i < mi.length; i++) {
				mi[i].setEnabled(false);
			}
		}
	}

	public void updateStatusbar(String text) {
		statusLine.setMessage(text);
	}

	public void lockWindow() {
		LOG.info("Locking");
		// menus[getFileMenu()].setEnabled(true);
		// menus[getAdministratorMenu()].setEnabled(false);
		// menus[getViewMenu()].setEnabled(false);
		// getSubMenu(getFileMenu(), "Lock workbench").setText("Unlock workbench");
		// locked = true;
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().setVisible(
				false);
	}

	public void unlockWindow() {
		LOG.info("Unlocking");
		// menus[getFileMenu()].setEnabled(true);
		// menus[getAdministratorMenu()].setEnabled(false);
		// menus[getViewMenu()].setEnabled(false);
		// getSubMenu(getFileMenu(), "Lock workbench").setText("Unlock workbench");
		// locked = true;
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().setVisible(
				true);
	}

	public void closeMainShell() {
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().close();
	}

	public boolean hasOpenWindows() {
		if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.getViewReferences().length > 0) {
			return true;
		} else {
			return false;
		}
	}



	

	private void setNurseMenus() {
		LOG.info("Setting nurse menus");
		menus[getFileMenu()].setEnabled(true);
		menus[getViewMenu()].setEnabled(true);
		getSubMenu(getViewMenu(), "Nurse calendar").setEnabled(true);
	}

	



	public void setNoMenus() {
		LOG.info("Setting no menus");
		menus[getFileMenu()].setEnabled(true);
		//menus[getAdministratorMenu()].setEnabled(false);
		menus[getViewMenu()].setEnabled(false);
		getSubMenu(getViewMenu(), "Nurse calendar").setEnabled(false);
		LOG.info("Finished setting no menus");
	}



	/*
	 * private void setViewNurseAppointmentMenus() { LOG.info("Setting view nurse
	 * appointments menus"); menus[getFileMenu()].setEnabled(true);
	 * menus[getViewMenu()].setEnabled(true);
	 * menus[getActionMenu()].setEnabled(true); getSubMenu(getViewMenu(), "View
	 * appointment requests").setEnabled(true); getSubMenu(getViewMenu(), "Search
	 * for appointment requests").setEnabled(true); }
	 */
	private void setSystemAdministratorMenus() {
		LOG.info("Setting system administrator menus");
		menus[getFileMenu()].setEnabled(true);
		menus[getAdministratorMenu()].setEnabled(true);
	}

	public IStatusLineManager getStatusLine() {
		return statusLine;
	}

	public void setStatusLine(IStatusLineManager statusLine) {
		this.statusLine = statusLine;
	}
}
