package za.co.momentum.recurring.nurse.client.reports;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import za.co.momentum.recurring.nurse.util.TNLogger;

public abstract class ExcelReport {
	protected HSSFWorkbook wb = null;
	protected HSSFSheet sheet = null;
	protected ExcelData data = null;;
	protected String filename = null;
	//All classes implementing this class must set the value of templateName
	protected String templateName = null;
	protected int rownum = 0;

	public ExcelReport(ExcelData data, String filename) {
		this.data = data;
		this.filename = filename;
	}

	public void writeExcellFile() {
		try {
			createWorkBook();
			writeReport();
			TNLogger.debug("Report written, now save try and Excel file");
			// Write the output to a file
			FileOutputStream fileOut = new FileOutputStream(filename);
			TNLogger.debug("FileOutputStream: " + fileOut);
			
			TNLogger.debug("About to write the Excel file...");
			wb.write(fileOut);
			fileOut.close();
		} catch (Exception e) {
			TNLogger.error("Error while saving excelfile " + e.getMessage());
			throw new RuntimeException(e);
		}
		TNLogger.debug("Done with excelfile " + filename);
	}

	protected abstract void writeReport();

	protected HSSFCell getCell(String bookmark) {
		for (Iterator rows = sheet.rowIterator(); rows.hasNext();) {
			HSSFRow row = (HSSFRow) rows.next();
			for (Iterator iter = row.cellIterator(); iter.hasNext();) {
				HSSFCell cell = (HSSFCell) iter.next();
				if (cell.getCellType() == HSSFCell.CELL_TYPE_STRING) {
					if (cell.getStringCellValue().equalsIgnoreCase(bookmark)) {
						rownum = row.getRowNum();
						return cell;
					}
				}
			}
		}
		return null;
	}
	
	protected HSSFCell setAlignment(HSSFCell cell, short alignment){
		HSSFCellStyle style =  cell.getCellStyle();
		style.setAlignment(alignment);
		cell.setCellStyle(style);
		return cell;
	}
	
	

	private void createWorkBook() {
		try {
			FileInputStream fi = new FileInputStream(templateName);
			wb = new HSSFWorkbook(fi);
			sheet = wb.getSheetAt(0);
		} catch (FileNotFoundException e) {
			TNLogger.error("Could not find template" + e.getMessage());
		} catch (IOException e) {
			TNLogger.error("IOException " + e.getMessage());
		}
	}
}
