package za.co.momentum.recurring.nurse.client.reports;

import java.io.IOException;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.eclipse.core.runtime.Platform;
import za.co.momentum.recurring.nurse.util.TNLogger;

public class NurseAppointmentReport extends ExcelReport {
	private String title;

	public NurseAppointmentReport(ExcelData data, String filename, String title) {
		super(data, filename);
		this.data = data;
		this.filename = filename;
		this.title = title;
		try {
			templateName = Platform.asLocalURL(
					Platform.getProduct().getDefiningBundle().getEntry(
							"nurseAppointmentsTemplate.xls")).getPath();
			TNLogger.debug("Template path = " + templateName);
		} catch (IOException e) {
			TNLogger.error("Could not find template contractedNursesTemplate.xls "
					+ e.getMessage());
		}
	}

	private void setStyle(HSSFCell c) {
		HSSFCellStyle style = c.getCellStyle();
		style.setAlignment(HSSFCellStyle.ALIGN_LEFT);
		style.setVerticalAlignment(HSSFCellStyle.VERTICAL_TOP);
		style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
		style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
		style.setBorderRight(HSSFCellStyle.BORDER_THIN);
		style.setBorderTop(HSSFCellStyle.BORDER_THIN);
		style.setWrapText(true);
		c.setCellStyle(style);
	}

	protected void writeReport() {
		HSSFCell c = getCell("$header");
		c.setCellValue(title);
		c = getCell("$appointments");
		HSSFCell temp = c;
		HSSFRow row = sheet.getRow(rownum);
		NurseReportDetail[] detail = data.getNurseReportDetail();
		for (int i = 0; i < detail.length; i++) {
			temp = row.createCell((short) 0);
			temp.setCellValue(detail[i].getAppointmentTime());
			setStyle(temp);
			temp = row.createCell((short) ((short) 1));
			temp.setCellValue(detail[i].getClientDetails());
			setStyle(temp);
			temp = row.createCell((short) ((short) 2));
			temp.setCellValue(detail[i].getAppointmentLocation());
			setStyle(temp);
			temp = row.createCell((short) ((short) 3));
			temp.setCellValue(detail[i].getRequirements());
			setStyle(temp);
			temp = row.createCell((short) ((short) 4));
			temp.setCellValue(detail[i].getClientContactDetails());
			setStyle(temp);
			row = sheet.createRow(row.getRowNum() + 1);
		}
	}
}
