package za.co.momentum.recurring.nurse.client.sao;

import java.net.ConnectException;
import java.util.Calendar;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.preferences.ConfigurationScope;
import org.eclipse.core.runtime.preferences.DefaultScope;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.osgi.service.prefs.Preferences;
import za.co.momentum.recurring.client.TNPlugin.Application;
import za.co.momentum.recurring.client.exceptions.UserCancelledException;
import za.co.momentum.recurring.nurse.client.ui.FastLanePreferencePage;
import za.co.momentum.recurring.nurse.client.ui.WebErrorWindow;
import za.co.momentum.recurring.nurse.client.ui.event.ServiceRetryListener;
import za.co.momentum.recurring.nurse.client.ui.tools.ContinuousProgressBar;
import za.co.momentum.recurring.nurse.client.ui.tools.ProgressAction;
import za.co.momentum.recurring.nurse.util.TNLogger;

/**
 * This class represents a service manager inside the client. It is an abstract
 * implementation of using specific services avaible inside the network.
 * 
 * 
 * @author NideBruyn Create on 8 Nov, 2005
 */
public abstract class AbstractServiceManager {
	// private static final int TIMEOUT = timeout;
	// Integer.parseInt(ClientProperties
	// .getInstance().getProperty(ClientProperties.CLIENT_TIMEOUT)); // sec
	private Object returnObject;
	private static final Logger LOG = Logger
			.getLogger(AbstractServiceManager.class);

	/**
	 * This method must be called when ever a ServiceAction needs to be executed.
	 * Incase a timeout has occured the returning object will be a
	 * TimeoutException.
	 * 
	 * @param serviceAction
	 * @return Object
	 */
	protected Object executeServiceAction(final ServiceAction serviceAction) {
		try {
			Preferences preferences = new ConfigurationScope()
					.getNode(Application.PLUGIN_ID);
			int tempTimeOut = preferences.getInt(FastLanePreferencePage.TIME_OUT, 0);
			if (tempTimeOut == 0) {
				IEclipsePreferences defaults = new DefaultScope()
						.getNode(Application.PLUGIN_ID);
				tempTimeOut = defaults.getInt(FastLanePreferencePage.TIME_OUT, 100);
			}
			final long TIMEOUT = tempTimeOut;
			final WebThread webCall = new WebThread(serviceAction);
			final long startTime = System.currentTimeMillis();
			final long endTS = System.currentTimeMillis() + TIMEOUT * 1000;
			ContinuousProgressBar progBar = new ContinuousProgressBar(PlatformUI
					.isWorkbenchRunning() ? PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell() : new Shell(), SWT.NULL,
					new ProgressAction() {
						public void executeAction() {
							webCall.start();
							while (webCall.isRunning() && System.currentTimeMillis() < endTS) {
								// Wait for return from webcall or timeout
								try {
									Thread.sleep(500);
								} catch (InterruptedException e) {
									LOG.debug("Interupt exception " + e.getMessage());
								}
							}
							LOG.debug("Web service profiling *********** duration="
									+ (Calendar.getInstance().getTimeInMillis() - startTime)
									+ " ms");
							LOG.debug("Web service profiling *********** object returned="
									+ returnObject);
						}
					});
			progBar.setHeadingDescription(Application.APPLICATION_NAME);
			progBar.setActionDescription("Busy, please wait . . .");
			progBar.open();
			if (progBar.isCancelled()) {
				returnObject = new UserCancelledException("User pressed cancelled");
			} else {
				returnObject = webCall.getReturnObject();
			}
			if (webCall.isRunning() && !progBar.isCancelled()) {
				// TimeOut occured
				TNLogger.debug("Timeout occured");
				// Do timeoutMethod
				// return new TimeoutException("Cannot wait any longer
				// sucker!");
				WebErrorWindow we = new WebErrorWindow();
				we.addServiceRetryListener(new ServiceRetryListener() {
					public void handleEvent() {
						executeServiceAction(serviceAction);
					}
				});
				we
						.setErrorText("A timeout error occurred. Please retry. \n \n"
								+ "Also make sure that the timeout setting is not too short.  It is recommended that the timeout setting be set to 20 seconds or longer.");
				we.open();
				// returnObject = null;
			}
//			if (returnObject instanceof AxisFault) {
//				AxisFault af = (AxisFault) returnObject;
//				if (af.getCause() instanceof ConnectException) {
//					// ErrorMessages.showMessage(ErrorMessages.M21);
//					return returnObject;
//				}
//			}
			LOG.debug("Return object from AbstractServiceManager =" + returnObject);
			return returnObject;
		} catch (Exception e) {
			LOG.error("Exception AbstractServiceManager very BAD " + e.getMessage());
			return e;
		}
	}
}
