package za.co.momentum.recurring.nurse.client.tools;

import ie.wombat.jbdiff.JBDiff;
import ie.wombat.jbdiff.JBUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.TreeMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.preferences.ConfigurationScope;
import org.eclipse.core.runtime.preferences.DefaultScope;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;
import org.eclipse.update.configuration.IConfiguredSite;
import org.eclipse.update.core.IArchiveReference;
import org.eclipse.update.core.ISite;
import org.eclipse.update.core.SiteManager;
import org.osgi.framework.Bundle;
import org.osgi.service.prefs.Preferences;

import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfStamper;

import za.co.momentum.recurring.client.TNPlugin.Application;
import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import za.co.momentum.recurring.nurse.client.model.ClientWindowManager;
import za.co.momentum.recurring.nurse.client.ui.FastLanePreferencePage;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.FastlaneFormBackup;
import za.co.momentum.recurring.nurse.util.adobe.AdobeUtils;

/**
 * Class used for different utilities regarding Pdf's
 * 
 * @author chdjager
 * 
 */
public class PdfFileManager {

	public static final String ECG_DIRECTORY = ClientProperties.getInstance()
			.getProperty(ClientProperties.ECG_DIRECTORY) + File.separator;

	public static final String PDF_SOURCE_DIRECTORY = ClientProperties
			.getInstance().getProperty(ClientProperties.PDF_SOURCE_DIRECTORY)
			+ File.separator;

	public static final String PDF_SAVE_DIRECTORY = ClientProperties
			.getInstance().getProperty(ClientProperties.PDF_SAVE_DIRECTORY)
			+ File.separator;

	// private static final String PDF_SENT_DIRECTORY =
	// ClientProperties.getInstance().getProperty(
	// ClientProperties.PDF_SENT_DIRECTORY)
	// + File.separator;

	private static final String AFRIKAANS = ClientProperties.getInstance()
			.getProperty(ClientProperties.AFRIKAANS);

	private static final String ENGLISH = ClientProperties.getInstance()
			.getProperty(ClientProperties.ENGLISH);

	public static final String AFRIKAANS_DIR = ClientProperties.getInstance()
			.getProperty(ClientProperties.AFRIKAANS_DIR) + File.separator;

	public static final String ENGLISH_DIR = ClientProperties.getInstance()
			.getProperty(ClientProperties.ENGLISH_DIR) + File.separator;

	private static final String PDF_NAME_CODE = ClientProperties.getInstance()
			.getProperty(ClientProperties.PDF_FILE_NAME_CODE);

	private static final String PDF_NAME_LENGTH = ClientProperties
			.getInstance().getProperty(ClientProperties.PDF_FILE_NAME_LENGTH);

	private static final Logger LOG = Logger.getLogger(PdfFileManager.class);

	private static final int BUFF_SIZE = 4096 * 8;

	private static final byte[] buffer = new byte[BUFF_SIZE];

	public PdfFileManager() {
	}

	/**
	 * Returns the installation directory path.
	 * 
	 * @return
	 */
	public static String getAbsoluteInstallPath() {
		String path = Platform.getInstallLocation().getURL().getPath();
		if (path.charAt(0) == '/') {
			path = path.substring(1, path.length());
		}
		return path;
	}

	public static String getWorkDirectory() {
		return getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY
				+ PDF_SAVE_DIRECTORY;
	}

	public static void backupAllFilesForClientNumber(String policyAlpha,
			String policyNumber, String clientName, String clientNumber) {

		FastlaneFormBackup fastlaneFormBackup = new FastlaneFormBackup();

		// Get all EForms in the client's folder
		String savedFilesDirectory = getAbsoluteInstallPath()
				+ PDF_SOURCE_DIRECTORY + PDF_SAVE_DIRECTORY + clientNumber;
		File clientEFormFolder = new File(savedFilesDirectory);
		if (!clientEFormFolder.exists()) {
			return;
		}

		File[] filesInClientFolder = clientEFormFolder.listFiles();
		for (int i = 0; i < filesInClientFolder.length; i++) {

			boolean backupSuccess = false;
			int counter = 0;

			// Create a backup file
			while (!backupSuccess) {
				if (counter == 5) {
					break;
				}
				// Try to create a backup of the file
				backupSuccess = fastlaneFormBackup.createFastlaneBackupForm(
						policyAlpha, policyNumber, clientName, clientNumber,
						filesInClientFolder[i], i);

				counter++;
			}
		}

	}

	private static void copyFile(String fromFile, String toFile) {
		// Copy the file to be edited to the save directory first before opening
		// it.
		FileInputStream fis = null;
		FileOutputStream fos = null;
		try {
			LOG.debug("Beginning copy process....");
			fis = new FileInputStream(fromFile);
			// Make the editable file
			File editableFile = new File(toFile);
			// Make all directories that may not exist.
			File parentDir = new File(editableFile.getParent());
			LOG.debug("Make directories");
			parentDir.mkdirs();
			LOG.debug("Create new file");
			editableFile.createNewFile();
			fos = new FileOutputStream(toFile);
			copy(fis, fos);
			LOG.debug("Done writing output stream.");
		} catch (FileNotFoundException e) {
			LOG.error("File not found: " + e);
			SendErrorEmail.sendErrorMail(e, "copyFile", "PdfFileManager");
		} catch (IOException e) {
			LOG.error("I/O problems copying pdf file: " + e);
			SendErrorEmail.sendErrorMail(e, "copyFile", "PdfFileManager");
		}
	}

	// /**
	// * Opens up a Pdf file. First a copy of the template Pdf is saved to
	// another directory, and then the new file is
	// * opened for editing. The directory where the file is created is named
	// according to the supplied client number, the
	// * client number is also inserted in the new file's name.
	// *
	// * @param clientNo
	// * @param fileName
	// * without .pdf extention
	// * @param language
	// * Language of the file to be opened.
	// */
	// public static void openPdfTemplate(String clientNo, String pdfFileNo,
	// String language) {
	//
	// String templatefilePath = getPDFFile(pdfFileNo, language.startsWith("A")
	// ? "A" : "E");
	//
	// LOG.debug("PDF FILE MANAGER - templatefilePath " + templatefilePath);
	//
	// String fileName = pdfFileNo + (language.startsWith("A") ? "A" : "E") +
	// ".pdf";
	// String sentfilePath = getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY +
	// PDF_SENT_DIRECTORY + clientNo + "/"
	// + clientNo + "_" + fileName;
	// LOG.debug("PDF FILE MANAGER - sentfilePath " + sentfilePath);
	//
	// String editablefilePath = getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY
	// + PDF_SAVE_DIRECTORY + clientNo + "/"
	// + clientNo + "_" + fileName;
	// LOG.debug("PDF FILE MANAGER - editablefilePath " + editablefilePath);
	//
	// Preferences preferences = new
	// ConfigurationScope().getNode(Application.PLUGIN_ID);
	// LOG.debug("Got preferences");
	//
	// String pdfEditorPath =
	// preferences.get(FastLanePreferencePage.PDF_EDITOR_PATH, "");
	//
	// try {
	// if (!checkIfFileIsSaved(clientNo, pdfFileNo, language.startsWith("A") ?
	// "A" : "E")) {
	// if (checkIfFileIsSent(clientNo, pdfFileNo, language.startsWith("A") ? "A"
	// : "E")) {
	// // Copy from sent to saved
	// LOG.debug("Start to copy file from sent to saved");
	// copyFile(sentfilePath, editablefilePath);
	// LOG.debug("Done copying file from sent to saved");
	// } else {
	// // copy from templates to saved
	// LOG.debug("Start to copy file from templates to saved");
	// if (templatefilePath != null) {
	// copyFile(templatefilePath, editablefilePath);
	// }
	// LOG.debug("Done copying file from templates to saved");
	// }
	// }
	// // Open the file
	// File testFile = new File(editablefilePath);
	// if (testFile.exists()) {
	// LOG.debug("PDF file opening .... " + editablefilePath);
	// if (pdfEditorPath.indexOf(".exe") == -1) {
	// IEclipsePreferences defaults = new
	// DefaultScope().getNode(Application.PLUGIN_ID);
	// pdfEditorPath = defaults.get(FastLanePreferencePage.PDF_EDITOR_PATH,
	// "default not set");
	// LOG.debug("got default pdfEditor path : " + pdfEditorPath);
	// }
	// LOG.debug("Show message popup");
	// MessageBox message = new
	// MessageBox(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
	// SWT.ICON_WORKING | SWT.OK);
	// message.setMessage("Please wait a while, selected Pdf file is being opened externally....");
	// message.setText("Message");
	// // message.open();
	// LOG.debug("Excecuting the runtime command for exe:" + pdfEditorPath + " "
	// + editablefilePath);
	// Runtime.getRuntime().exec(pdfEditorPath + " " + editablefilePath);
	// LOG.debug("PDF file " + editablefilePath + " opened for editing.");
	// } else {
	// MessageBox message = new
	// MessageBox(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
	// SWT.ICON_ERROR | SWT.OK);
	// message.setMessage("PDF file could not be found. Cannot open " +
	// pdfFileNo);
	// message.setText("Message");
	// message.open();
	// }
	// } catch (IOException ex) {
	// MessageBox message = new
	// MessageBox(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
	// SWT.ICON_ERROR | SWT.OK);
	// message
	// .setMessage("Could not find Pdf Editor.\n\n"
	// +
	// "Please go to File -> Preferences -> Fastlane Preferences -> Acrobat Application Path and Executable\n"
	// +
	// "and Browse to your local Acrobat.exe file (eg. C:\\Program Files\\Adobe\\Acrobat 7.0\\Acrobat\\Acrobat.exe)");
	// message.setText("Message");
	// message.open();
	// LOG.error(" Error opening PDF FILE (IOException) " + ex);
	// ex.printStackTrace();
	// } catch (Exception e) {
	// e.printStackTrace();
	// SendErrorEmail.sendErrorMail(e, "openPdfTemplate", "PdfFileManager");
	// LOG.error(" Error opening PDF FILE" + e);
	// }
	// }

	/**
	 * Opens up a Pdf file. First a copy of the template Pdf is saved to another
	 * directory, and then the new file is opened for editing. The directory
	 * where the file is created is named according to the supplied client
	 * number, the client number is also inserted in the new file's name.
	 * 
	 * @param clientNo
	 * @param fileName
	 *            without .pdf extention
	 * @param language
	 *            Language of the file to be opened.
	 */
	public static void openPdfTemplate(String clientNo, String pdfFileNo,
			String language, long appClientId) {

		// Check if work directory exists
		String workDirectory = getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY
				+ PDF_SAVE_DIRECTORY;
		File workDirFile = new File(workDirectory);
		if (!workDirFile.exists()) {
			if (!workDirFile.mkdirs()) {
				MessageBox message = new MessageBox(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell(), SWT.ICON_ERROR
						| SWT.OK);
				message.setMessage("Work directory could not be created. Please investigate! Directory location:\n"
						+ workDirFile);
				message.setText("Error creating work directory");
				message.open();
			}
		}

		String templatefilePath = getBlankPdfTemplateFile(pdfFileNo,
				language.startsWith("A") ? "A" : "E");
		LOG.debug("PDF FILE MANAGER - templatefilePath " + templatefilePath);

		String fileName = pdfFileNo + (language.startsWith("A") ? "A" : "E")
				+ ".pdf";
		String editablefilePath = getAbsoluteInstallPath()
				+ PDF_SOURCE_DIRECTORY + PDF_SAVE_DIRECTORY + clientNo + "/"
				+ clientNo + "_" + fileName;
		LOG.debug("PDF FILE MANAGER - editablefilePath " + editablefilePath);

		Preferences preferences = new ConfigurationScope()
				.getNode(Application.PLUGIN_ID);
		String pdfEditorPath = preferences.get(
				FastLanePreferencePage.PDF_EDITOR_PATH, "");

		try {
			String formStatus = getFileStatus(clientNo, pdfFileNo,
					language.startsWith("A") ? "A" : "E");
			if (formStatus.equalsIgnoreCase("Form not found in work folder")) {
				// copy from templates to saved
				if (templatefilePath != null) {
					copyFile(templatefilePath, editablefilePath);
				}
				LOG.debug("Done copying file from templates to saved");
			}

			// Open the file
			File testFile = new File(editablefilePath);
			if (testFile.exists()) {
				LOG.debug("PDF file opening ....  " + editablefilePath);
				if ("windows".equalsIgnoreCase(System.getProperty("os.name"))) {
					if (pdfEditorPath.indexOf(".exe") == -1) {
						IEclipsePreferences defaults = new DefaultScope()
								.getNode(Application.PLUGIN_ID);
						pdfEditorPath = defaults.get(
								FastLanePreferencePage.PDF_EDITOR_PATH,
								"default not set");
					}
				}

				// MessageBox message = new
				// MessageBox(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
				// SWT.ICON_WORKING | SWT.OK);
				// message.setMessage("Please wait a while, selected Pdf file is being opened externally....");
				// message.setText("Message");
				// message.open();

				// AdobeUtils.prepareForPrepopulation();
				// new ProcessBuilder(pdfEditorPath, editablefilePath).start();

				String fileOpenPath;
				if (formStatus
						.equalsIgnoreCase("Form not found in work folder")
						|| formStatus
								.equalsIgnoreCase("Blank (or unsaved) form")) {
					fileOpenPath = AdobeUtils.generateFilledFdf(appClientId,
							clientNo, editablefilePath);
				} else {
					fileOpenPath = editablefilePath;
				}
				new ProcessBuilder(pdfEditorPath, fileOpenPath).start();

			} else {
				MessageBox message = new MessageBox(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell(), SWT.ICON_ERROR
						| SWT.OK);
				message.setMessage("PDF file could not be found in work directory. Cannot open "
						+ pdfFileNo + "\n\nFile location:\n" + editablefilePath + "\n\nTemplate location:\n" + templatefilePath);
				message.setText("Message");
				message.open();
			}
		} catch (IOException ex) {
			MessageBox message = new MessageBox(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), SWT.ICON_ERROR
					| SWT.OK);
			message.setMessage("Could not find Pdf Editor.\n\n"
					+ "Please go to File -> Preferences -> Fastlane Preferences -> Acrobat Application Path and Executable\n"
					+ "and Browse to your local Acrobat.exe file (eg. C:\\Program Files\\Adobe\\Acrobat 7.0\\Acrobat\\Acrobat.exe)");
			message.setText("Message");
			message.open();

			ex.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
			SendErrorEmail
					.sendErrorMail(e, "openPdfTemplate", "PdfFileManager");
			LOG.error("  Error opening PDF FILE" + e);
		}
	}

	/**
	 * Creates a zip file
	 * 
	 * @param fileName
	 *            to be zipped
	 */
	public static File zipFile(String fileName) {
		File tmp = null;
		InputStream is = null;
		ZipOutputStream out = null;
		try {
			File file = new File(fileName);
			if (file.exists()) {
				String zipFileName = file.getAbsolutePath();
				zipFileName = zipFileName
						.substring(0, zipFileName.length() - 3) + "zip";
				// create the zip file on the file system
				LOG.debug("PDF FILE MANAGER - ZIPFILE NAME  " + zipFileName);
				tmp = new File(zipFileName);
				tmp.createNewFile();
				// create a ZipFile with this name
				is = new FileInputStream(file);
				out = new ZipOutputStream(new FileOutputStream(zipFileName));
				out.putNextEntry(new ZipEntry(file.getName()));
				// Get the size of the file
				long length = file.length();
				// You cannot create an array using a long type.
				// It needs to be an int type.
				// Before converting to an int type, check
				// to ensure that file is not larger than Integer.MAX_VALUE.
				if (length > Integer.MAX_VALUE) {
					throw new DomainRuntimeException("File too large to zip");
					// File is too large
				}
				// Create the byte array to hold the data
				byte[] bytes = new byte[(int) length];
				// Read in the bytes
				int offset = 0;
				int numRead = 0;
				while (offset < bytes.length
						&& (numRead = is.read(bytes, offset, bytes.length
								- offset)) >= 0) {
					offset += numRead;
				}
				// Ensure all the bytes have been read in
				if (offset < bytes.length) {
					throw new IOException("Could not completely read file "
							+ file.getName());
				}
				// Close the input stream and return bytes
				out.write(bytes); // byte array of file
			}
		} catch (Exception e) {
			LOG.error("Problems zipping the file " + fileName + " -- " + e);
			e.printStackTrace();
			SendErrorEmail.sendErrorMail(e, "zipFile", "PdfFileManager");
		} finally {
			try {
				if (is != null) {
					is.close();
					LOG.debug("Inputstream closed");
				}
				if (out != null) {
					out.closeEntry();
					out.close();
					LOG.debug("Outputstream closed");
				}
			} catch (IOException e) {
				LOG.debug("Error closing streams : " + e);
			}
		}
		return tmp;
	}

	/**
	 * Copies the file to the sent directory. Used after successful sending to
	 * server of files.
	 * 
	 * @param clientNo
	 * @param fileNo
	 * @param language
	 */
	// public static void copyFileToSent(String clientNo, String formNo) {
	// if (formNo != null) {
	// if (formNo.length() > 0) {
	// try {
	// String savedFilesDirectory = getAbsoluteInstallPath() +
	// PDF_SOURCE_DIRECTORY + PDF_SAVE_DIRECTORY + clientNo;
	// File savedFilesDir = new File(savedFilesDirectory);
	//
	// if (savedFilesDir.isDirectory()) {
	// File[] savedFiles = savedFilesDir.listFiles();
	// for (int i = 0; i < savedFiles.length; i++) {
	// File sourceFile = savedFiles[i];
	//
	// // Check if clientNo and formNo combination in filename, do not worry
	// about language :)
	// String formIdentifier = clientNo + "_" + formNo;
	// if (sourceFile.getName().indexOf(formIdentifier) != -1) {
	// String sentFilePath = getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY +
	// PDF_SENT_DIRECTORY + clientNo
	// + "/" + sourceFile.getName();
	//
	// LOG.debug("PDF FILE MANAGER - source file path: " +
	// sourceFile.getAbsolutePath());
	// LOG.debug("PDF FILE MANAGER - target file path: " + sentFilePath);
	//
	// FileInputStream fis = null;
	// FileOutputStream fos = null;
	// try {
	// fis = new FileInputStream(sourceFile);
	// File sentFile = new File(sentFilePath);
	//
	// File parentDir = new File(sentFile.getParent());
	// parentDir.mkdirs();
	//
	// sentFile.createNewFile();
	// fos = new FileOutputStream(sentFilePath);
	// copy(fis, fos);
	//
	// sourceFile.delete();
	//
	// } catch (FileNotFoundException e) {
	// LOG.error("File not found: " + e);
	// SendErrorEmail.sendErrorMail(e, "copyFileToSent", "PdfFileManager");
	// } catch (IOException e) {
	// LOG.error("I/O problems copying pdf file: " + e);
	// SendErrorEmail.sendErrorMail(e, "copyFileToSent", "PdfFileManager");
	// }
	// }
	// }
	// // If dir is empty delete
	// if (savedFilesDir.list().length == 0) {
	// savedFilesDir.delete();
	// }
	// }
	// } catch (Exception e) {
	// LOG.error(e);
	// SendErrorEmail.sendErrorMail(e, "copyFileToSent", "PdfFileManager");
	// }
	// } else {
	// LOG.error("Problems copying the file " + formNo +
	// " to sent dir -- File name cannot be empty");
	// }
	// } else {
	// LOG.error("Problems copying the file " + formNo +
	// " to sent dir -- File name cannot be null");
	// }
	// }
	private static void copy(InputStream in, OutputStream out)
			throws IOException {
		try {
			while (true) {
				synchronized (buffer) {
					int amountRead = in.read(buffer);
					if (amountRead == -1) {
						break;
					}
					out.write(buffer, 0, amountRead);
				}
			}
		} finally {
			if (out != null) {
				out.flush();
				out.close();
			}
			if (in != null) {
				in.close();
			}
		}
	}

	/**
	 * Retrieves all the pdf docs names in Plugins according to the language.
	 * (Used for additional pdf selection)
	 * 
	 * @param language
	 * @return list of pdf's
	 */
	public static ArrayList getAllPdfFileNameInDir(String language) {
		ArrayList listOfDocs = new ArrayList();
		language = (language.startsWith("A") ? "A" : "E").toUpperCase();

		try {
			IConfiguredSite[] sites = SiteManager.getLocalSite()
					.getCurrentConfiguration().getConfiguredSites();
			ISite localSite = sites[0].getSite();
			if (localSite.getPluginEntryCount() == 0) {
				if (sites.length > 1) {
					localSite = sites[1].getSite();
				}
			}
			IArchiveReference[] archives = localSite.getArchives();
			LOG.debug("Getting Pdf Plugins ...");
			for (int i = 0; i < archives.length; i++) {
				IArchiveReference archive = archives[i];
				String path = archive.getURL().getPath();
				if (path.indexOf(PDF_NAME_CODE) != -1) {
					File f = new File(archive.getURL().getPath());
					String name = f.getName();
					int pdfNameLength = Integer.parseInt(PDF_NAME_LENGTH);
					name = name.substring(0, pdfNameLength);
					LOG.debug("Plugin name = " + name);

					String description = getFormDescription(name,
							language.startsWith("A") ? "A" : "E");

					name = description + " (UNDERW" + getFormNo(name) + ") ";
					listOfDocs.add(name);
					LOG.debug("Pdf plugin found & name added :" + name
							+ (language.startsWith("A") ? "A" : "E"));
				}
			}

			if (listOfDocs.isEmpty()) {
				for (String formName : FORM_NAMES) {
					Bundle bundle = Platform.getBundle(formName);
					if (bundle != null) {
						listOfDocs.add(getFormDescription(formName,
								language.startsWith("A") ? "A" : "E")
								+ " (UNDERW" + getFormNo(formName) + ") ");
					}
				}
			}
		} catch (CoreException e) {
			LOG.error("Cannot retrieve Pdf plugins: " + e.getMessage());
			SendErrorEmail.sendErrorMail(e, "getAllPdfFileNameInDir",
					"PdfFileManager");
		}
		return listOfDocs;
	}

	private static final String[] FORM_NAMES = { "UWFORM107", "UWFORM000",
			"UWFORM001", "UWFORM002", "UWFORM003", "UWFORM004", "UWFORM006",
			"UWFORM020", "UWFORM021", "UWFORM022", "UWFORM024", "UWFORM025",
			"UWFORM027", "UWFORM028", "UWFORM029", "UWFORM030", "UWFORM031",
			"UWFORM032", "UWFORM033", "UWFORM034", "UWFORM035", "UWFORM036",
			"UWFORM038", "UWFORM039", "UWFORM041", "UWFORM044", "UWFORM049",
			"UWFORM105" };

	public static String getBlankPdfTemplateFile(String fileNo, String language) {

		String filename = "";

		try {
			String pluginName = "UWFORM" + fileNo;
			filename = fileNo + (language.startsWith("A") ? "A" : "E") + ".pdf";
			Bundle bundle = Platform.getBundle(pluginName);

			if (bundle != null) {
				URL url = Platform.asLocalURL(bundle.getEntry(filename));
				LOG.debug("Requesting pdf file " + url.getPath());
				return url.getPath();
			}

			return null;

		} catch (IOException e) {
			LOG.error("Could not get file " + filename + " Exception = "
					+ e.getMessage());
			SendErrorEmail.sendErrorMail(e, "getPDFFile", "ClientProperties");
			return null;
		} catch (Exception ex) {
			ex.printStackTrace();
			SendErrorEmail.sendErrorMail(ex, "getPDFFile", "ClientProperties");
			return null;
		}
	}

	// public static String getPDFFileByNumber(String fileNo, String language) {
	//
	// try {
	// String pluginName = "UWFORM" + fileNo;
	// String filename = fileNo + (language.startsWith("A") ? "A" : "E") +
	// ".pdf";
	//
	// Bundle bundle = Platform.getBundle(pluginName);
	// if (bundle != null) {
	// URL url = Platform.asLocalURL(bundle.getEntry(filename));
	// LOG.debug("Requesting pdf file " + url.getPath());
	// return url.getPath();
	// }
	//
	// return null;
	//
	// } catch (IOException e) {
	// SendErrorEmail.sendErrorMail(e, "getPDFFile", "ClientProperties");
	// return null;
	// } catch (Exception ex) {
	// ex.printStackTrace();
	// SendErrorEmail.sendErrorMail(ex, "getPDFFile", "ClientProperties");
	// return null;
	// }
	// }

	/**
	 * Retrieves all the pdf doc names in the save directory according to the
	 * language.
	 * 
	 * @param language
	 * @return list of pdf's
	 */
	// public static ArrayList getAllPdfDocNamesInSaveAndSentDir(String
	// clientNo) {
	// ArrayList listOfDocs = new ArrayList();
	// // For save dir
	// String templatefilePath = getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY
	// + PDF_SAVE_DIRECTORY + clientNo + "/";
	// LOG.debug("PDF FILE MANAGER - getAllPdfDocsInDir template path " +
	// templatefilePath);
	// File dir = new File(templatefilePath);
	// if (dir.isDirectory()) {
	// LOG.debug("PDF FILE MANAGER - getAllPdfDocsInDir path " + dir.getPath());
	// File[] list = dir.listFiles();
	// for (int i = 0; i < list.length; i++) {
	// File tempFile = list[i];
	// if (tempFile.getName().indexOf(".pdf") != -1) {
	// listOfDocs.add(tempFile.getName());
	// }
	// }
	// }
	// // For Sent dir
	// templatefilePath = getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY +
	// PDF_SENT_DIRECTORY + clientNo + "/";
	// LOG.debug("PDF FILE MANAGER - getAllPdfDocsInDir template path " +
	// templatefilePath);
	// dir = new File(templatefilePath);
	// if (dir.isDirectory()) {
	// LOG.debug("PDF FILE MANAGER - getAllPdfDocsInDir path " + dir.getPath());
	// File[] list = dir.listFiles();
	// for (int i = 0; i < list.length; i++) {
	// File tempFile = list[i];
	// if (tempFile.getName().indexOf(".pdf") != -1) {
	// if (!listOfDocs.contains(tempFile.getName())) {
	// listOfDocs.add(tempFile.getName());
	// }
	// }
	// }
	// }
	// return listOfDocs;
	// }
	/**
	 * Retrieves all the pdf doc names in the sent directory according to the
	 * language.
	 * 
	 * @param language
	 * @return list of pdf's
	 */
	// public static ArrayList getAllPdfDocNamesInSentDir(String clientNo) {
	// ArrayList listOfDocs = new ArrayList();
	// String templatefilePath = getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY
	// + PDF_SENT_DIRECTORY + "/" + clientNo
	// + "/";
	// LOG.debug("PDF FILE MANAGER - getAllPdfDocsInDir template path " +
	// templatefilePath);
	// File dir = new File(templatefilePath);
	// if (dir.isDirectory()) {
	// LOG.debug("PDF FILE MANAGER - getAllPdfDocsInDir path " + dir.getPath());
	// File[] list = dir.listFiles();
	// for (int i = 0; i < list.length; i++) {
	// File tempFile = list[i];
	// if (tempFile.getName().indexOf(".pdf") != -1) {
	// listOfDocs.add(tempFile.getName());
	// }
	// }
	// }
	// return listOfDocs;
	// }
	/**
	 * Deletes all the zipped files in a directory. Used to clear after
	 * compressing and sending to server
	 * 
	 * @param clientNo
	 */
	public static void deleteZipFiles(String clientNo) {
		try {
			String applicationDirectory = getAbsoluteInstallPath()
					+ PDF_SOURCE_DIRECTORY + PDF_SAVE_DIRECTORY + clientNo;
			File saveDir = new File(applicationDirectory);
			File[] list = saveDir.listFiles();

			for (int i = 0; i < list.length; i++) {
				File tempFile = list[i];
				if (tempFile.getName().indexOf(".zip") != -1) {
					LOG.debug("PDF FILE MANAGER - " + tempFile.getName()
							+ " deleted = " + tempFile.delete());
				}
			}
		} catch (Exception e) {
			// LOG.error("Cannot delete the zip file " + e.getMessage());
			// SendErrorEmail.sendErrorMail(e, "deleteZipFiles",
			// "PdfFileManager");
		}
	}

	/**
	 * Converts a file into a byte array.
	 * 
	 * @param fileName
	 *            as string
	 * @return the byte array
	 */
	public static byte[] getByteArrayForFile(String clientNo, String fileName,
			String language) {
		return getByteArrayForFile(getFileFromClientDetails(clientNo, fileName,
				language.startsWith("A") ? "A" : "E"));
	}

	/**
	 * Get File object from client details.
	 * 
	 * @param fileName
	 *            as string
	 * @return the byte array
	 */
	public static File getFileFromClientDetailsNotZipped(String clientNo,
			String fileName, String language) {
		return getFileFromClientDetailsImpl(clientNo, fileName,
				language.startsWith("A") ? "A" : "E", false);
	}

	public static File getFileFromClientDetails(String clientNo,
			String fileName, String language) {
		return getFileFromClientDetailsImpl(clientNo, fileName,
				language.startsWith("A") ? "A" : "E", true);
	}

	private static File getFileFromClientDetailsImpl(String clientNo,
			String fileName, String language, boolean zipped) {

		if (fileName != null) {
			if (fileName.length() > 0) {
				fileName = fileName + (language.startsWith("A") ? "A" : "E")
						+ ".pdf";
				String editablefilePath = getAbsoluteInstallPath()
						+ PDF_SOURCE_DIRECTORY + PDF_SAVE_DIRECTORY + clientNo
						+ "/" + clientNo + "_" + fileName;
				if (zipped) {
					File file = zipFile(editablefilePath);
					return file;
				} else {
					return new File(editablefilePath);
				}
			} else {
				LOG.error("Problems converting the file " + fileName
						+ " to byte array -- File name cannot be empty");
			}
		} else {
			LOG.error("Problems converting the file " + fileName
					+ " to byte array -- File name cannot be null");
		}
		return null;
	}

	/**
	 * Converts a file into a byte array.
	 * 
	 * @param file
	 * @return the byte array
	 */
	public static byte[] getByteArrayForFile(File file) {
		InputStream is = null;
		if (file != null) {
			if (file.exists()) {
				// Get the size of the file
				long length = file.length();
				// Create the byte array to hold the data
				byte[] bytes = new byte[(int) length];
				try {
					is = new FileInputStream(file);
					// Read in the bytes
					int offset = 0;
					int numRead = 0;
					while (offset < bytes.length
							&& (numRead = is.read(bytes, offset, bytes.length
									- offset)) >= 0) {
						offset += numRead;
					}
					return bytes;

				} catch (Exception e) {
					LOG.error("Problems converting the file " + file.getName()
							+ " to byte array -- " + e);
					SendErrorEmail.sendErrorMail(e, "getByteArrayForFile",
							"PdfFileManager");
				} finally {
					if (is != null) {
						try {
							is.close();
						} catch (IOException ignored) {
						}
					}
				}
			}
		} else {
			LOG.error("File is null cannot convert " + file.getName()
					+ " to byte array");
		}
		return null;
	}

	/**
	 * Checks if the file is in the sent directory.
	 * 
	 * @param clientNo
	 * @param fileName
	 * @param language
	 * @return
	 */
	// public static boolean checkIfFileIsSent(String clientNo, String fileName,
	// String language) {
	// fileName = fileName + (language.startsWith("A") ? "A" : "E") + ".pdf";
	// // Get the correct directory according to language of client
	// if (language.equals(AFRIKAANS)) {
	// language = AFRIKAANS_DIR;
	// } else if (language.equals(ENGLISH)) {
	// language = ENGLISH_DIR;
	// }
	// String editablefilePath = getAbsoluteInstallPath() + PDF_SOURCE_DIRECTORY
	// + PDF_SENT_DIRECTORY + "/" + clientNo
	// + "/" + clientNo + "_" + fileName;
	// LOG.debug("checkIfFileIsSent " + editablefilePath);
	// File testFile = new File(editablefilePath);
	// return testFile.exists();
	// }

	/**
	 * Gets the status of the file in the client work folder
	 * 
	 * @param clientNo
	 * @param fileName
	 * @param language
	 * @return String fileStatus
	 */
	public static String getFileStatus(String clientNo, String fileNo,
			String language) {

		try {
			String fileName = fileNo + (language.startsWith("A") ? "A" : "E")
					+ ".pdf";
			// Get the correct directory according to language of client
			String completedFilePath = getAbsoluteInstallPath()
					+ PDF_SOURCE_DIRECTORY + PDF_SAVE_DIRECTORY + "/"
					+ clientNo + "/" + clientNo + "_" + fileName;
			File completedFile = new File(completedFilePath);
			File completedFDF = new File(completedFilePath.replace(".pdf",
					".fdf"));

			if (completedFile.exists() && completedFDF.exists()) {
				// Check if the form has same checksum as template (to prevent
				// blank forms)
				long completedFormChecksum = JBUtil.getChecksum(completedFile)
						.longValue();

				// Get template file and checksum
				String templateFilePath = getBlankPdfTemplateFile(fileNo,
						language);
				File templateFile = new File(templateFilePath);
				long templateFormChecksum = JBUtil.getChecksum(templateFile)
						.longValue();

				if (completedFormChecksum != templateFormChecksum) {
					return "Form completed";
				} else {
					return "Blank (or unsaved) form";
				}
			} else {
				return "Form not found in work folder";
			}
		} catch (Exception e) {
			LOG.error(e);
			return "Form status unknown";
		}
	}

	public static String getFormNo(String formName) {
		String subs = "";
		try {
			if (formName.indexOf("UNDERW") != -1) {
				subs = formName.substring(formName.indexOf("UNDERW") + 6,
						formName.indexOf("UNDERW") + 9);

			} else if (formName.indexOf("UWFORM") != -1) {
				subs = formName.substring(formName.indexOf("UWFORM") + 6,
						formName.indexOf("UWFORM") + 9);

			} else if (formName.indexOf("_") != -1) {
				int _place = formName.indexOf("_");
				subs = formName.substring(_place + 1, _place + 4);

			} else {
				return formName;
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
		return subs;
	}

	public static byte[] getEcgBytes(File ecgFile) {

		return getByteArrayForFile(ecgFile);
	}

	private static String getLatestEcgFileNameForClient(String clientNumber) {
		// Get list of available ecg files in ecg folder
		File ecgFolder = new File(getAbsoluteInstallPath()
				+ PDF_SOURCE_DIRECTORY + ECG_DIRECTORY);
		File[] files = ecgFolder.listFiles();

		TreeMap ecgTree = new TreeMap();

		for (int i = 0; i < files.length; i++) {
			if (files[i].getName().indexOf(clientNumber) > 0) {
				ecgTree.put(new Long(files[i].lastModified()), files[i]);
			}
		}
		Long lastKey = (Long) ecgTree.lastKey();
		File ecgFile = (File) ecgTree.get(lastKey);

		return null;
	}

	private static String getFileNameWithoutExtension(String fileName) {
		File tmpFile = new File(fileName);
		tmpFile.getName();
		int whereDot = tmpFile.getName().lastIndexOf('.');
		if (0 < whereDot && whereDot <= tmpFile.getName().length() - 2) {
			return tmpFile.getName().substring(0, whereDot);
			// extension = filename.substring(whereDot+1);
		}
		return "";
	}

	private static String getFormDescription(String name, String language) {

		// TODO - Change PDF File descriptions here

		String description = "";

		if (language.toLowerCase().startsWith("a")) {

			if (name.equalsIgnoreCase("UWFORM107")) {
				description = "Fingerprick form";
			}

			if (name.equalsIgnoreCase("UWFORM000")) {
				description = "Multiply Individual Health Assessment";
			}
			if (name.equalsIgnoreCase("UWFORM001")) {
				description = "Vertroulike mediese verslag";
			}
			if (name.equalsIgnoreCase("UWFORM002")) {
				description = "Inspanningselektrokardiogram (EKG) vir lewensversekeringsdoeleindes";
			}
			if (name.equalsIgnoreCase("UWFORM003")) {
				description = "Versoek vir ondersoek deur 'n patoloog of laboratorium";
			}
			if (name.equalsIgnoreCase("UWFORM004")) {
				description = "Longfunksietoets vir lewensversekering";
			}
			if (name.equalsIgnoreCase("UWFORM006")) {
				description = "Vertroulike kort mediese verslag";
			}
			if (name.equalsIgnoreCase("UWFORM020")) {
				description = "Spesiale hartverslag";
			}
			if (name.equalsIgnoreCase("UWFORM021")) {
				description = "Hertoets van bloeddruk";
			}
			if (name.equalsIgnoreCase("UWFORM022")) {
				description = "Vraelys: Diabetes";
			}
			if (name.equalsIgnoreCase("UWFORM024")) {
				description = "Geestesgesondheidsvraelys";
			}
			if (name.equalsIgnoreCase("UWFORM025")) {
				description = "Myriad: Gewig, lengte en bloeddruklesing";
			}
			if (name.equalsIgnoreCase("UWFORM027")) {
				description = "Vraelys: Borskaspyn";
			}
			if (name.equalsIgnoreCase("UWFORM028")) {
				description = "Diabetesvraelys";
			}
			if (name.equalsIgnoreCase("UWFORM029")) {
				description = "Gastrontestinalevraelys";
			}
			if (name.equalsIgnoreCase("UWFORM030")) {
				description = "Asemhalingsvraelys";
			}
			if (name.equalsIgnoreCase("UWFORM031")) {
				description = "Nek- en rugpynvraelys";
			}
			if (name.equalsIgnoreCase("UWFORM032")) {
				description = "Epilepsievraelys";
			}
			if (name.equalsIgnoreCase("UWFORM033")) {
				description = "Geestesgesondheidsvraelys";
			}
			if (name.equalsIgnoreCase("UWFORM034")) {
				description = "Aanvullende vraelys";
			}
			if (name.equalsIgnoreCase("UWFORM035")) {
				description = "Verklaring van gesondheid";
			}
			if (name.equalsIgnoreCase("UWFORM036")) {
				description = "Vraelys oor die gebruik van middele";
			}
			if (name.equalsIgnoreCase("UWFORM038")) {
				description = "Gewrigte vraelys";
			}
			if (name.equalsIgnoreCase("UWFORM039")) {
				description = "Artritis vraelys";
			}
			if (name.equalsIgnoreCase("UWFORM041")) {
				description = "Vraelys vir selektiewe lewens";
			}
			if (name.equalsIgnoreCase("UWFORM044")) {
				description = "Bevestiging van u gesondheid";
			}
			if (name.equalsIgnoreCase("UWFORM049")) {
				description = "COVID-19: Herstel vraelys";
			}

			if (name.equalsIgnoreCase("UWFORM105")) {
				description = "COVID-19 risiko vrae";
			}

		} else {

			if (name.equalsIgnoreCase("UWFORM107")) {
				description = "Fingerprick form";
			}
			if (name.equalsIgnoreCase("UWFORM000")) {
				description = "Multiply Individual Health Assessment";
			}
			if (name.equalsIgnoreCase("UWFORM001")) {
				description = "Confidential medical report";
			}
			if (name.equalsIgnoreCase("UWFORM002")) {
				description = "Effort electrocardiogram (ECG) for life insurance purposes";
			}
			if (name.equalsIgnoreCase("UWFORM003")) {
				description = "Request for examination by a pathologist or laboratory";
			}
			if (name.equalsIgnoreCase("UWFORM004")) {
				description = "Lung function test for life insurance";
			}
			if (name.equalsIgnoreCase("UWFORM006")) {
				description = "Confidential short medical report";
			}
			if (name.equalsIgnoreCase("UWFORM020")) {
				description = "Special heart report";
			}
			if (name.equalsIgnoreCase("UWFORM021")) {
				description = "Retest of blood pressure";
			}
			if (name.equalsIgnoreCase("UWFORM022")) {
				description = "Diabetes questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM024")) {
				description = "Mental health questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM025")) {
				description = "Weight, height and blood pressure reading";
			}
			if (name.equalsIgnoreCase("UWFORM027")) {
				description = "Chest pain questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM028")) {
				description = "Diabetes questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM029")) {
				description = "Gastro-intestinal questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM030")) {
				description = "Respiratory questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM031")) {
				description = "Neck and back pain questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM032")) {
				description = "Epilepsy questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM033")) {
				description = "Personal wellbeing questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM034")) {
				description = "Supplementary questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM035")) {
				description = "Declaration of health";
			}
			if (name.equalsIgnoreCase("UWFORM036")) {
				description = "Substance use questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM038")) {
				description = "Joint questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM039")) {
				description = "Arthritis questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM041")) {
				description = "Questionnaire for select lives";
			}
			if (name.equalsIgnoreCase("UWFORM044")) {
				description = "Health confirmation";
			}
			if (name.equalsIgnoreCase("UWFORM049")) {
				description = "COVID-19: Recovery questionnaire";
			}
			if (name.equalsIgnoreCase("UWFORM105")) {
				description = "COVID-19 risk questionnaire";
			}

		}
		return description;
	}

	public static void openBlankPdf(String pdfFileNo, String formName,
			String language) {

		String templatefilePath = getBlankPdfTemplateFile(pdfFileNo,
				language.startsWith("A") ? "A" : "E");

		String saveFilePath = getSaveFilePath(formName);
		if (saveFilePath == null) {
			return;
		}

		Preferences preferences = new ConfigurationScope()
				.getNode(Application.PLUGIN_ID);
		String pdfEditorPath = preferences.get(
				FastLanePreferencePage.PDF_EDITOR_PATH, "");

		try {

			// Copy blank form to selected save location
			copyFile(templatefilePath, saveFilePath);

			// Open the file
			File newFile = new File(saveFilePath);
			if (newFile.exists()) {

				if (pdfEditorPath.indexOf(".exe") == -1) {
					IEclipsePreferences defaults = new DefaultScope()
							.getNode(Application.PLUGIN_ID);
					pdfEditorPath = defaults.get(
							FastLanePreferencePage.PDF_EDITOR_PATH,
							"default not set");
				}

				// AdobeUtils.prepareForPrepopulation();
				new ProcessBuilder(pdfEditorPath, saveFilePath).start();

			} else {
				MessageBox message = new MessageBox(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell(), SWT.ICON_ERROR
						| SWT.OK);
				message.setMessage("PDF file could not be found. Cannot open "
						+ pdfFileNo);
				message.setText("Message");
				message.open();
			}
		} catch (IOException ex) {
			MessageBox message = new MessageBox(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), SWT.ICON_ERROR
					| SWT.OK);
			message.setMessage("Could not open Pdf Editor. \n"
					+ "Please go to File -> Preferences -> Fastlane Preferences -> Acrobat Application Path and Executable\n"
					+ "and Browse to your local Acrobat.exe file (eg. C:\\Program Files\\Adobe\\Acrobat 7.0\\Acrobat\\Acrobat.exe)");
			message.setText("Message");
			message.open();
			LOG.error("  Error opening PDF FILE (IOException) " + ex);
			ex.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
			SendErrorEmail
					.sendErrorMail(e, "openPdfTemplate", "PdfFileManager");
			LOG.error("  Error opening PDF FILE" + e);
		}
	}

	private static String getSaveFilePath(String formName) {

		FileDialog fileSaver = new FileDialog(ClientWindowManager.getInstance()
				.getShell(), SWT.SAVE);
		fileSaver.setText("Please select a location and enter a file name");
		fileSaver.setFilterPath(PdfFileManager.getAbsoluteInstallPath()
				+ PdfFileManager.PDF_SOURCE_DIRECTORY);
		fileSaver.setFileName(formName);
		fileSaver.setFilterExtensions(new String[] { "*.pdf" });
		fileSaver.setFilterNames(new String[] { "PDF Files" + " (pdf)" });

		String fileName = fileSaver.open();
		if (fileName == null) {
			return null;
		}

		return fileName;
	}

	public static void removeFileFromWork(String clientNumber, String formNo) {
		if (formNo != null) {
			if (formNo.length() > 0) {
				try {
					String savedFilesDirectory = getAbsoluteInstallPath()
							+ PDF_SOURCE_DIRECTORY + PDF_SAVE_DIRECTORY
							+ clientNumber;
					File savedFilesDir = new File(savedFilesDirectory);

					if (savedFilesDir.isDirectory()) {
						File[] savedFiles = savedFilesDir.listFiles();
						for (int i = 0; i < savedFiles.length; i++) {
							File sourceFile = savedFiles[i];

							// Check if clientNo and formNo combination in
							// filename, do not worry about language :)
							String formIdentifier = clientNumber + "_" + formNo;
							if (sourceFile.getName().indexOf(formIdentifier) != -1) {
								try {
									sourceFile.delete();
								} catch (Exception e) {
									e.printStackTrace();
								}
							}
						}
						// If dir is empty delete
						if (savedFilesDir.list().length == 0) {
							savedFilesDir.delete();
						}
					}
				} catch (Exception e) {
					LOG.error(e);
				}
			}
		}
	}

	private static boolean flattenPdfFile(String pathToPdf, String pathToFlatPdf) {

		try {
			PdfReader pdfReader = new PdfReader(pathToPdf);
			PdfStamper pdfStamper = new PdfStamper(pdfReader,
					new FileOutputStream(pathToFlatPdf));

			pdfStamper.setFormFlattening(true);
			pdfStamper.close();

			return true;

		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	private static byte[] getDiffFromFiles(File blankTemplatePdfFile,
			File completedPdfFile) {

		try {
			byte[] diffBytes = JBDiff.bsdiff(blankTemplatePdfFile,
					completedPdfFile);
			return diffBytes;

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public static byte[] getDiffFromFlattenedPdfFiles(
			String completedPdfFilePath, String templatePdfFilePath) {

		File flatCompletedPdfFile = null;
		try {
			// First try to flatten the completed Pdf form
			String flatCompletedPdfPath = completedPdfFilePath + "_flat.pdf";
			if (flattenPdfFile(completedPdfFilePath, flatCompletedPdfPath)) {
				flatCompletedPdfFile = new File(flatCompletedPdfPath);

				// Now try to flatten the template Pdf form
				String flatTemplatePdfPath = templatePdfFilePath + "_flat.pdf";
				if (flattenPdfFile(templatePdfFilePath, flatTemplatePdfPath)) {

					// Lastly get the diff between the flattened forms
					File flatTemplatePdfFile = new File(flatTemplatePdfPath);

					byte[] flattenedFilesDiff = getDiffFromFiles(
							flatTemplatePdfFile, flatCompletedPdfFile);
					if (flattenedFilesDiff != null) {
						return flattenedFilesDiff;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (flatCompletedPdfFile.exists()) {
				flatCompletedPdfFile.delete();
			}
		}
		return null;
	}

	public static byte[] getDiffFromPdfFiles(String completedPdfFilePath,
			String templatePdfFilePath) {

		try {
			File templateFile = new File(templatePdfFilePath);
			File completedFile = new File(completedPdfFilePath);

			byte[] flattenedFilesDiff = getDiffFromFiles(templateFile,
					completedFile);
			if (flattenedFilesDiff != null) {
				return flattenedFilesDiff;
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	public static boolean createFileFromBytes(File file, byte[] fileBytes) {

		FileOutputStream fos;

		try {
			fos = new FileOutputStream(file);
			fos.write(fileBytes);
			fos.flush();
			fos.close();

		} catch (FileNotFoundException e) {
			return false;

		} catch (IOException e) {
			return false;
		}

		return true;
	}

}
