package za.co.momentum.recurring.nurse.client.ui;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class EventBroker {

	private List softObservers = new ArrayList(); // Sequence is important!
	private List formalObservers = new ArrayList();
	
	/**
	 * Soft oberservers implement no specific interface or extends no particular class,
	 * the event broker will look for a method called notifyMe(Object o) and if found,
	 * call it.
	 * 
	 * 
	 * @param observer
	 */
	
	
	public void addSoftObserver(Object observer) {
		if (!isAValidSoftObserver(observer.getClass())) {
			throw new RuntimeException("SoftObserver does not implement notifyMe(Object)");
		}
		softObservers.add(observer);
	}
	public void addFormalObserver(EventObserver observer) {
		formalObservers.add(observer);
	}
	
	
	public void notifyObservers(Object o) {
		for (Iterator iter = softObservers.iterator(); iter.hasNext();) {
			Object element = (Object) iter.next();
			callObserver(element.getClass(), o);
			
		}
		for (Iterator iter = formalObservers.iterator(); iter.hasNext();) {
			EventObserver element = (EventObserver) iter.next();
			element.notifyFormal(o);
			
		}
		
	}
	
	private Method getNotifyMeMethod(Class cl) throws NoSuchMethodException {
		
		return cl.getMethod("notifyMe", new Class[] {Object.class});
		
	}
	private boolean isAValidSoftObserver(Class cl) {
		try {
			getNotifyMeMethod(cl);
		} catch (NoSuchMethodException e) {
			return false;
		}
		return true;
	}
	private void callObserver(Class cl, Object arg0) {
		Method mm = null;
		try {
			mm = getNotifyMeMethod(cl);
		} catch (NoSuchMethodException e) {

			throw new RuntimeException(e); // This is propably too harsh, msg and return may be better, these are soft observers after all.
		}
		
		Object [] args = new Object [] {arg0};
		try {
			mm.invoke(this, args);
		} catch (Exception e) {

			throw new RuntimeException(e); // This is propably too harsh, msg and return may be better, these are soft observers after all.
		}
		
	}
}
