package za.co.momentum.recurring.nurse.client.update;

import java.net.URL;
import java.util.HashSet;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.update.configuration.IConfiguredSite;
import org.eclipse.update.core.ICategory;
import org.eclipse.update.core.IFeature;
import org.eclipse.update.core.IFeatureReference;
import org.eclipse.update.core.ISite;
import org.eclipse.update.core.ISiteFeatureReference;
import org.eclipse.update.core.SiteManager;
import org.eclipse.update.operations.IInstallFeatureOperation;
import org.eclipse.update.search.IQueryUpdateSiteAdapter;
import org.eclipse.update.search.IUpdateSearchCategory;
import org.eclipse.update.search.IUpdateSearchFilter;
import org.eclipse.update.search.IUpdateSearchQuery;
import org.eclipse.update.search.IUpdateSearchResultCollector;
import org.eclipse.update.search.UpdateSearchRequest;
import org.eclipse.update.search.UpdateSearchScope;
import org.eclipse.update.standalone.InstallCommand;
//import org.eclipse.update.ui.UpdateJob;

import za.co.momentum.recurring.client.TNPlugin.Application;
import za.co.momentum.recurring.nurse.client.ui.tools.ContinuousProgressBar;
import za.co.momentum.recurring.nurse.client.ui.tools.ProgressAction;
import za.co.momentum.recurring.nurse.util.ClientProperties;

public class UpdateAction extends Action implements IAction {
  private static final Logger LOG = Logger.getLogger(UpdateAction.class);

  private String homebase = ClientProperties.getInstance().getProperty(ClientProperties.UPDATE_URL);

  private boolean restart = false;

  private boolean doUpdate = false;

  private boolean exitApp = false;

  public boolean isRestart() {
    return restart;
  }

  public boolean isExitApp() {
    return exitApp;
  }

  public UpdateAction() {
    setId(homebase);
    setText("&Update...");
    setToolTipText("Search for updates for " + Application.APPLICATION_NAME);
  }

  public void run() {

    System.out.println("Update process starting...");
    ContinuousProgressBar progBar = new ContinuousProgressBar(PlatformUI.isWorkbenchRunning() ? PlatformUI
        .getWorkbench().getActiveWorkbenchWindow().getShell() : new Shell(), SWT.NULL, new ProgressAction() {

      public void executeAction() {
        try {
          UpdateSearchScope scope = new UpdateSearchScope();
          scope.setFeatureProvidedSitesEnabled(true);
          scope.addSearchSite("Travel Nurse Update site", new URL(homebase + "site.xml"), null);
          UpdateSearchRequest searchRequest = new UpdateSearchRequest(new IUpdateSearchCategory() {
            class Query implements IUpdateSearchQuery {
              public void run(ISite site, String[] categoriesToSkip, IUpdateSearchFilter filter,
                  IUpdateSearchResultCollector collector, IProgressMonitor monitor) {
                ISiteFeatureReference[] refs = site.getFeatureReferences();
                HashSet ignores = new HashSet();
                if (categoriesToSkip != null) {
                  for (int i = 0; i < categoriesToSkip.length; i++) {
                    ignores.add(categoriesToSkip[i]);
                  }
                }
                monitor.beginTask("", refs.length); //$NON-NLS-1$
                for (int i = 0; i < refs.length; i++) {
                  ISiteFeatureReference ref = refs[i];
                  boolean skipFeature = false;
                  if (monitor.isCanceled())
                    break;
                  if (ignores.size() > 0) {
                    ICategory[] categories = ref.getCategories();
                    for (int j = 0; j < categories.length; j++) {
                      ICategory category = categories[j];
                      if (ignores.contains(category.getName())) {
                        skipFeature = true;
                        break;
                      }
                    }
                  }
                  try {
                    if (!skipFeature) {
                      if (filter.accept(ref)) {
                        IFeature feature = ref.getFeature(null);
                        if (filter.accept(feature))
                          collector.accept(feature);
                        monitor.subTask(feature.getLabel());
                      }
                    }
                  } catch (CoreException e) {

                  } finally {
                    monitor.worked(1);
                  }
                }
              }

              /*
               * (non-Javadoc)
               * 
               * @see org.eclipse.update.internal.ui.search.ISearchQuery#getSearchSite()
               */
              public IQueryUpdateSiteAdapter getQuerySearchSite() {
                return null;
              }
            }

            private IUpdateSearchQuery[] queries;

            public String getId() {
              return "org.eclipse.update.core.unified-search";
            }

            public void setId(String id) {
              // 
            }

            public IUpdateSearchQuery[] getQueries() {
              queries = new IUpdateSearchQuery[] { new Query() };
              return queries;
            }
          }, new UpdateSearchScope());
          // searchRequest.addFilter(new BackLevelFilter());
          searchRequest.setScope(scope);
          /*UpdateJob job = new UpdateJob("Updating ", searchRequest);
          job.setUser(true);
          job.setPriority(Job.INTERACTIVE);
          job.schedule();
          while ((job.getState() == Job.WAITING) || (job.getState() == Job.RUNNING)) {
            Thread.sleep(300);
          }
          IInstallFeatureOperation[] ops = job.getUpdates();

          for (int i = 0; i < ops.length; i++) {
            IFeature feature = ops[i].getFeature();
            if (checkFeature(feature.getVersionedIdentifier().getIdentifier(), feature.getVersionedIdentifier()
                .getVersion().toString())) {

              // Code here to ask User if update should commence, or quit app
              displayUpdateMsg();
              if (!doUpdate) {
                exitApp = true;
                return;
              }

              LOG.debug("Installing " + feature.getVersionedIdentifier().getIdentifier() + " with version "
                  + feature.getVersionedIdentifier().getVersion().toString());
              InstallCommand command = new InstallCommand(feature.getVersionedIdentifier().getIdentifier(), feature
                  .getVersionedIdentifier().getVersion().toString(), homebase, null, "false");
              boolean answer = command.run();
              LOG.debug("Did update = " + answer);
              command.applyChangesNow();
              restart = answer;
            }
          }*/
        } catch (Exception e) {
          LOG.debug("Exception " + e.getMessage());
        }
      }
    });
    progBar.setHeadingDescription(Application.APPLICATION_NAME);
    progBar.setActionDescription("Checking for updates . . .");
    progBar.run();

    if (exitApp == true) {
      return;
    }
  }

  private void displayUpdateMsg() {

    doUpdate = false;
    try {
      Display.getDefault().syncExec(new Runnable() {
        public void run() {

          Display display = Display.getCurrent();
          MessageBox messageBox = new MessageBox(new Shell(display), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
          messageBox.setText("Update found for Fastlane MedPerson application");
          String message = "A new update is available for this application. Please make sure that you have a reliable connection\n"
              + "to the Momentum network before continuing.\n\nDo you want to continue with the update? (Selecting NO will exit\n"
              + "the application)";

          messageBox.setMessage(message);

          int selectionId = messageBox.open();
          if (selectionId == SWT.YES) {
            doUpdate = true;
          }

        }
      });

    } catch (Exception e) {
      e.printStackTrace();
      doUpdate = false;
    }
  }

  private boolean checkFeature(String feature, String version) {
    IConfiguredSite[] sites;
    boolean canUpdate = false;
    try {
      sites = SiteManager.getLocalSite().getCurrentConfiguration().getConfiguredSites();
      for (int x = 0; x < sites.length; x++) {
        ISite localSite = sites[x].getSite();
        IFeatureReference[] features = localSite.getCurrentConfiguredSite().getConfiguredFeatures();
        for (int i = 0; i < features.length; i++) {
          LOG.debug("Feature on this site " + features[i].getName());
          ISiteFeatureReference localFeature = (ISiteFeatureReference) features[i];
          if (localFeature.getVersionedIdentifier().getIdentifier().equalsIgnoreCase(feature)) {
            LOG.debug("Local feature = " + localFeature.getVersionedIdentifier().getIdentifier());
            LOG.debug("Remote feature = " + feature);
            LOG.debug("Local version = " + localFeature.getVersionedIdentifier().getVersion());
            LOG.debug("Remote version = " + version);
            LOG.debug("Compare method returns "
                + localFeature.getVersionedIdentifier().getVersion().toString().compareTo(version));
            if (localFeature.getVersionedIdentifier().getVersion().toString().compareTo(version) != 0) {
              canUpdate = true;
            }
          }
        }
      }
    } catch (CoreException e) {
      LOG.error("Exception on UPDATE = " + e.getMessage());
      LOG.fatal("Error above should not happen");
    }
    return canUpdate;
  }
}
