/*
 * Momentum Risk & Savings IT
 * Project: Myriad
 * 
 * Created on Apr 19, 2011
 *
 */
package za.co.momentum.recurring.nurse.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Map.Entry;

public class FastlaneFormBackup {

  public static final String POLICY_ALPHA = "POLICYALPHA";

  public static final String POLICY_NUMBER = "POLICYNUMBER";

  public static final String CLIENT_NUMBER = "CLIENTNUMBER";

  public static final String CLIENT_NAME = "CLIENTNAME";

  public static final String ORIGINAL_FILENAME = "ORIGINALFILENAME";

  public static final String BACKUP_FILENAME = "BACKUPFILENAME";

  public static final String BACKUP_DATE = "BACKUPDATE";

  public static final String INFO = "INFO";

  private static final String EFORM_BACKUP_DIRECTORY = ClientProperties.getInstance().getProperty(
      ClientProperties.EFORM_BACKUP_DIRECTORY);

  public boolean createFastlaneBackupForm(String policyAlpha, String policyNumber, String clientName,
      String clientNumber, File fastlaneEFormFile, int fileCounter) {

    try {
      // Get EForm byte array
      // File fastlaneEFormFile = new File(eFormLocation);
      byte[] eFormBytes = getBytesFromFile(fastlaneEFormFile);

      // Create backup file name which will also be used in descriptor
      String backupFileName = createBackupFileName(fileCounter) + ".flb";

      // Get the file description details
      String fileDescription = createFileDescriptor(policyAlpha, policyNumber, clientName, clientNumber,
          fastlaneEFormFile.getName(), backupFileName);

      // Get byte array of policy description string
      byte[] fileDescriptionBytes = fileDescription.getBytes();

      // Merge byte arrays (Add file descriptions to top of file byte array)
      byte[] newBytes = mergeBytes(fileDescriptionBytes, eFormBytes);

      // Check if backup dir exists and create if not
      File backupLocationDir = new File(EFORM_BACKUP_DIRECTORY);
      if (!backupLocationDir.exists()) {
        String main = EFORM_BACKUP_DIRECTORY.substring(0, EFORM_BACKUP_DIRECTORY.lastIndexOf('/'));
        File mainDir = new File(main);
        if (mainDir.exists()) {
          backupLocationDir.mkdir();
        } else {
          if (mainDir.mkdir()) {
            backupLocationDir.mkdir();
          }
        }
      }

      // Create the backup file
      File fastlaneBackupFile = new File(EFORM_BACKUP_DIRECTORY + File.separator + backupFileName);

      // Create new file from bytes in backup location
      createFileFromBytes(fastlaneBackupFile, newBytes);

      // Check if the backup file is there
      if (!fastlaneBackupFile.exists()) {
        return false;
      }

      // If all is ok
      return true;

    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }

  public Map getAllBackupFileDescriptors() {

    Map listOfFileDescriptors = new TreeMap(Collections.reverseOrder());
    // Map listOfFileDescriptors = new TreeMap();

    // Check if backup dir exists and create if not
    File backupLocationDir = new File(EFORM_BACKUP_DIRECTORY);
    if (!backupLocationDir.exists()) {
      String main = EFORM_BACKUP_DIRECTORY.substring(0, EFORM_BACKUP_DIRECTORY.lastIndexOf('/'));
      File mainDir = new File(main);
      if (mainDir.exists()) {
        backupLocationDir.mkdir();
      } else {
        if (mainDir.mkdir()) {
          backupLocationDir.mkdir();
        }
      }
    }

    File[] fastlaneBackupFiles = backupLocationDir.listFiles();
    if (fastlaneBackupFiles == null || fastlaneBackupFiles.length == 0) {
      return listOfFileDescriptors;
    }

    for (int i = 0; i < fastlaneBackupFiles.length; i++) {
      byte[] fastlaneBackupFileBytes = getBytesFromFile(fastlaneBackupFiles[i]);
      String backupFileDescriptor = getDescriptorFromBackupFile(fastlaneBackupFileBytes);
      if (backupFileDescriptor.equalsIgnoreCase("")) {
        continue;
      }
      listOfFileDescriptors.put(fastlaneBackupFiles[i].getName(), backupFileDescriptor);
    }

    return listOfFileDescriptors;
  }

  public Map getBackupFileDescriptorsOfLastXDays(int xDays) {

    Map listOfFileDescriptors = new TreeMap(Collections.reverseOrder());

    // Check if backup dir exists and create if not
    File backupLocationDir = new File(EFORM_BACKUP_DIRECTORY);
    if (!backupLocationDir.exists()) {
      String main = EFORM_BACKUP_DIRECTORY.substring(0, EFORM_BACKUP_DIRECTORY.lastIndexOf('/'));
      File mainDir = new File(main);
      if (mainDir.exists()) {
        backupLocationDir.mkdir();
      } else {
        if (mainDir.mkdir()) {
          backupLocationDir.mkdir();
        }
      }
    }

    // Get start and end date for files to retrieve and setup a filter
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
    Calendar now = Calendar.getInstance();
    String endDate = sdf.format(now.getTime());
    now.add(Calendar.DATE, -xDays);
    String startDate = sdf.format(now.getTime());
    FileFilterDateIntervalUtils filter = new FileFilterDateIntervalUtils(startDate, endDate);

    File[] fastlaneBackupFiles = backupLocationDir.listFiles(filter);
    if (fastlaneBackupFiles == null || fastlaneBackupFiles.length == 0) {
      return listOfFileDescriptors;
    }

    for (int i = 0; i < fastlaneBackupFiles.length; i++) {
      byte[] fastlaneBackupFileBytes = getBytesFromFile(fastlaneBackupFiles[i]);
      String backupFileDescriptor = getDescriptorFromBackupFile(fastlaneBackupFileBytes);
      if (backupFileDescriptor.equalsIgnoreCase("")) {
        continue;
      }
      listOfFileDescriptors.put(fastlaneBackupFiles[i].getName(), backupFileDescriptor);
    }

    return listOfFileDescriptors;
  }

  public void removeBackupsOlderThanXDays(int xDays) {

    try {
      // Check if backup dir exists and create if not
      File backupLocationDir = new File(EFORM_BACKUP_DIRECTORY);
      if (!backupLocationDir.exists()) {
        String main = EFORM_BACKUP_DIRECTORY.substring(0, EFORM_BACKUP_DIRECTORY.lastIndexOf('/'));
        File mainDir = new File(main);
        if (mainDir.exists()) {
          backupLocationDir.mkdir();
        } else {
          if (mainDir.mkdir()) {
            backupLocationDir.mkdir();
          }
        }
      }

      // Setup a filter
      SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
      Calendar now = Calendar.getInstance();

      // Get the End Date
      now.add(Calendar.DATE, -xDays);
      String endDate = sdf.format(now.getTime());

      // Get the Start Date (Clean all files from epoch)
      String startDate = sdf.format(new Date(0));
      FileFilterDateIntervalUtils filter = new FileFilterDateIntervalUtils(startDate, endDate);

      File[] fastlaneBackupFiles = backupLocationDir.listFiles(filter);
      if (fastlaneBackupFiles == null || fastlaneBackupFiles.length == 0) {
        return;
      }

      // Now loop through all files returned and delete them
      for (int i = 0; i < fastlaneBackupFiles.length; i++) {
        try {
          fastlaneBackupFiles[i].delete();
        } catch (Exception e) {
          e.printStackTrace();
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public String getPropertyFromDescriptor(String fastlaneBackupFileDescriptor, String property) {

    StringTokenizer st = new StringTokenizer(fastlaneBackupFileDescriptor, "|");
    while (st.hasMoreTokens()) {
      String descriptorToken = st.nextToken();
      if (descriptorToken.startsWith(property)) {
        int startIndex = descriptorToken.indexOf("=");
        return descriptorToken.substring(startIndex + 1, descriptorToken.length());
      }
    }
    return "";
  }

  public byte[] getEFormFileBytesFromBackupFile(String backupFileName) {

    // Get handle on file
    File fastlaneEFormBackupFile = new File(EFORM_BACKUP_DIRECTORY + File.separator + backupFileName);
    if (!fastlaneEFormBackupFile.exists()) {
      return null;
    }

    // Get bytes of backup file
    byte[] backupFileBytes = getBytesFromFile(fastlaneEFormBackupFile);

    // Create new array to store EForm in
    long eFormBytesSize = fastlaneEFormBackupFile.length() - 300;
    byte[] eFormFileBytes = new byte[intValue(eFormBytesSize)];

    System.arraycopy(backupFileBytes, 300, eFormFileBytes, 0, eFormFileBytes.length);

    return eFormFileBytes;
  }

  private String createFileDescriptor(String policyAlpha, String policyNumber, String clientName, String clientNumber,
      String fileName, String backupFileName) {

    // Get backup date
    Calendar now = Calendar.getInstance();
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    String backupDate = sdf.format(new Date(now.getTimeInMillis()));

    // Create policy description string
    StringBuffer policyDescription = new StringBuffer();
    policyDescription.append("POLICYALPHA=");
    policyDescription.append(policyAlpha);
    policyDescription.append("|");
    policyDescription.append("POLICYNUMBER=");
    policyDescription.append(policyNumber);
    policyDescription.append("|");
    policyDescription.append("CLIENTNAME=");
    policyDescription.append(clientName);
    policyDescription.append("|");
    policyDescription.append("CLIENTNUMBER=");
    policyDescription.append(clientNumber);
    policyDescription.append("|");
    policyDescription.append("ORIGINALFILENAME=");
    policyDescription.append(fileName);
    policyDescription.append("|");
    policyDescription.append("BACKUPFILENAME=");
    policyDescription.append(backupFileName);
    policyDescription.append("|");
    policyDescription.append("BACKUPDATE=");
    policyDescription.append(backupDate);
    policyDescription.append("|");
    policyDescription.append("INFO=");
    policyDescription.append("");
    policyDescription.append("|");

    // Pad policy description with blanks until 300 long
    while (policyDescription.length() < 300) {
      policyDescription.append(" ");
    }

    return policyDescription.toString();
  }

  private String createBackupFileName(int fileCounter) {
    Calendar now = Calendar.getInstance();
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
    String backupFileName = sdf.format(new Date(now.getTimeInMillis())) + fileCounter;
    return backupFileName;
  }

  private byte[] mergeBytes(byte[] policyDescriptionBytes, byte[] eFormBytes) {
    byte[] newFileBytes = new byte[policyDescriptionBytes.length + eFormBytes.length];
    System.arraycopy(policyDescriptionBytes, 0, newFileBytes, 0, policyDescriptionBytes.length);
    System.arraycopy(eFormBytes, 0, newFileBytes, policyDescriptionBytes.length, eFormBytes.length);

    return newFileBytes;
  }

  private String getDescriptorFromBackupFile(byte[] fastlaneBackupFileBytes) {

    byte[] descriptorBytes = new byte[300];
    for (int i = 0; i < 300; i++) {
      descriptorBytes[i] = fastlaneBackupFileBytes[i];
    }

    String backupFileDescriptor = new String(descriptorBytes);

    // Make sure this is a valid Fastlane EForm backup file
    if (!backupFileDescriptor.startsWith(POLICY_ALPHA)) {
      return "";
    }
    return backupFileDescriptor;
  }

  private byte[] getBytesFromFile(File file) {

    FileInputStream fileInputStream = null;
    byte[] bytes = new byte[(int) file.length()];
    try {
      fileInputStream = new FileInputStream(file);
      fileInputStream.read(bytes);

    } catch (Exception e) {
      throw new RuntimeException(e);

    } finally {
      try {
        fileInputStream.close();
      } catch (Exception e) {
      }
    }
    return bytes;
  }

  private boolean createFileFromBytes(File file, byte[] fileBytes) {

    FileOutputStream fos;

    try {
      fos = new FileOutputStream(file);
      fos.write(fileBytes);
      fos.flush();
      fos.close();

    } catch (FileNotFoundException e) {
      return false;

    } catch (IOException e) {
      return false;
    }

    return true;
  }

  private int intValue(long value) {
    int valueInt = (int) value;
    if (valueInt != value) {
      throw new IllegalArgumentException("The long value " + value + " is not within range of the int type");
    }
    return valueInt;
  }

  public static void main(String[] args) {
    new FastlaneFormBackup().run();
  }

  private void run() {

    File testBackupFile = new File(
        "C:\\Fastlane2\\MedPerson\\pdfSource\\savedFiles\\1234567890123\\1234567890123_UNDERW027.pdf");

    createFastlaneBackupForm("", "200100084", "Riaan Goddard", "1234567890123", testBackupFile, 1);

    Map backupFileDescriptors = getAllBackupFileDescriptors();

    Iterator entries = backupFileDescriptors.entrySet().iterator();
    while (entries.hasNext()) {
      Entry thisEntry = (Entry) entries.next();
      String key = (String) thisEntry.getKey();
      String backupFileDescriptor = (String) thisEntry.getValue();

      String policyAlpha = getPropertyFromDescriptor(backupFileDescriptor, FastlaneFormBackup.POLICY_ALPHA);
      System.out.println("Policy Alpha = " + policyAlpha);
      String policyNumber = getPropertyFromDescriptor(backupFileDescriptor, FastlaneFormBackup.POLICY_NUMBER);
      System.out.println("Policy Number = " + policyNumber);
      String clientNumber = getPropertyFromDescriptor(backupFileDescriptor, FastlaneFormBackup.CLIENT_NUMBER);
      System.out.println("Client Number = " + clientNumber);
      String backupDate = getPropertyFromDescriptor(backupFileDescriptor, FastlaneFormBackup.BACKUP_DATE);
      System.out.println("Backup Date = " + backupDate);
      String backupFileName = getPropertyFromDescriptor(backupFileDescriptor, FastlaneFormBackup.BACKUP_FILENAME);
      System.out.println("Backup File Name = " + backupFileName);

      byte[] eFormBytes = getEFormFileBytesFromBackupFile(backupFileName);

      File eFormFile = new File("C:\\Fastlane2 Backup\\Forms\\test.pdf");
      createFileFromBytes(eFormFile, eFormBytes);
    }
  }
}
