package za.co.momentum.recurring.nurse.util.adobe;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.preferences.ConfigurationScope;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;

import za.co.momentum.recurring.client.TNPlugin.Application;
import za.co.momentum.recurring.nurse.client.model.ClientWindowManager;
import za.co.momentum.recurring.nurse.client.ui.FastLanePreferencePage;
import za.co.momentum.recurring.nurse.util.DBHelper;
import za.co.momentum.risk.fastlane.nurse.eforms.FormDTO;
import za.co.momentum.risk.fastlane.nurse.services.eform.EFormService;
import za.co.momentum.risk.fastlane.nurse.services.eform.EFormServiceRest;

import com.samskivert.mustache.Mustache;
import com.samskivert.mustache.Template;

public final class AdobeUtils {

	private AdobeUtils() {
	}

	private static Logger logger = Logger.getLogger(AdobeUtils.class);

	private static class AdobeDetail {
		String folderName;
		String registryName;
		String fileName;

		public AdobeDetail(String folderName, String registryName,
				String fileName) {
			this.folderName = folderName;
			this.registryName = registryName;
			this.fileName = fileName;
		}
	}

	// in future we won't need Adobe or need to improve the search heuristics
	public static void detectAndSaveInstallPath() throws Exception {
		IEclipsePreferences preferences = ConfigurationScope.INSTANCE
				.getNode(Application.PLUGIN_ID);

		String currentPath = preferences.get(
				FastLanePreferencePage.PDF_EDITOR_PATH, "");
		logger.info("checking if Adobe path is valid");
		if (new File(currentPath).exists() && !currentPath.contains("AcroRd32")) { // TODO:
																					// allow
																					// Reader
																					// later
			logger.info("Adobe's path is valid: " + currentPath);
			return;
		}
		logger.info("Adobe's path is invalid");

		List<AdobeDetail> adobeDetails = new ArrayList<AdobeDetail>();
		adobeDetails.add(new AdobeDetail("Acrobat", "Adobe Acrobat",
				"Acrobat.exe"));
		adobeDetails.add(new AdobeDetail("Reader", "Acrobat Reader",
				"AcroRd32.exe"));

		logger.info("detecting latest the Adobe path from registry");
		String bestPath = searchRegistry(adobeDetails);

		if (bestPath == null) {
			logger.info("could not detect the latest Adobe path from registry");
			logger.info("detecting latest the Adobe path from filesystem");
			bestPath = searchFilesystem(adobeDetails);
		}

		if (bestPath == null) {
			logger.warn("could not detect the latest Adobe path from filesystem");
			bestPath = "";
		} else {
			logger.info("best detected Adobe path: " + bestPath);
		}

		preferences.put(FastLanePreferencePage.PDF_EDITOR_PATH, bestPath);
		preferences.flush();
	}

	private static String searchRegistry(List<AdobeDetail> adobeDetails)
			throws IllegalAccessException, InvocationTargetException {
		for (AdobeDetail adobeDetail : adobeDetails) {
			for (int i = 25; i > 0; i--) {
				String registryPath = WinRegistry.readString(
						WinRegistry.HKEY_CURRENT_USER, "Software\\Adobe\\"
								+ adobeDetail.registryName + "\\" + i
								+ ".0\\InstallPath", "");

				File expectedPath = new File(registryPath + "\\"
						+ adobeDetail.fileName);
				if (expectedPath.exists()) {
					return expectedPath.getAbsolutePath();
				}
			}
		}

		return null;
	}

	private static String searchFilesystem(List<AdobeDetail> adobeDetails) {
		for (AdobeDetail adobeDetail : adobeDetails) {
			for (int i = 25; i > 0; i--) {
				String adobePortion = "\\Adobe\\" + adobeDetail.folderName
						+ " " + i + ".0\\" + adobeDetail.folderName + "\\"
						+ adobeDetail.fileName;

				File expectedPath = new File(System.getenv("ProgramFiles")
						+ adobePortion);
				if (expectedPath.exists()) {
					return expectedPath.getAbsolutePath();
				}

				expectedPath = new File(System.getenv("ProgramFiles(x86)")
						+ adobePortion);
				if (expectedPath.exists()) {
					return expectedPath.getAbsolutePath();
				}
			}
		}

		return null;
	}

	public static void prepareForPrepopulation() {
		try {
			Runtime.getRuntime()
					.exec("REG ADD \"HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings\" /v GlobalUserOffline /t REG_DWORD /d 0 /f");
		} catch (IOException e) {
			logger.warn("unable to set GlobalUserOffline in registry", e);
		}
	}

	public static String generateFilledFdf(long appClientId, String clientNo,
			String pdfPath) {
		try {
			logger.debug("creating flattened form");
			FlattenedForm form = getFlattenedForm(appClientId,
					new File(pdfPath).getName());
			logger.debug("created flattened form");

			logger.debug("reading FDF");
			InputStream is = AdobeUtils.class
					.getResourceAsStream("template.fdf");
			logger.debug("got InputStream");
			InputStreamReader isr = new InputStreamReader(is);
			logger.debug("got InputStreamReader");
			BufferedReader br = new BufferedReader(isr);
			logger.debug("got BufferedReader");
			Template mustache = Mustache.compiler().defaultValue("")
					.compile(br);
			logger.debug("filled template");
			br.close();
			isr.close();
			is.close();
			logger.debug("created template to populate");

			String outputPath = pdfPath.replace(".pdf", ".fdf");
			PrintWriter output = new PrintWriter(new File(outputPath));
			mustache.execute(form, output);
			output.close();
			logger.debug("FDF written to: " + outputPath);

			return outputPath;

		} catch (Exception e) {
			logger.error("failed to generate FDF", e);
			return pdfPath;
		}
	}

	private static FlattenedForm getFlattenedForm(long appClientId,
			String filePath) {
		FormDTO formData = null;

		try {
			logger.debug("calling webservice for form data");
			EFormService service = EFormServiceRest.getInstance();

			formData = service.getFormData(appClientId);
			logger.debug("successfully got form data from webservice");

			DBHelper.AddEForm(formData, appClientId);
			logger.debug("saved form data to DB");
		} catch (Exception e) {
			logger.error("Error " + e.getMessage());
		}

		if (formData == null) {
			try {
				formData = (FormDTO) DBHelper.getForm(appClientId,
						DBHelper.FullData);
				if (formData != null) {
					logger.debug("retrieved form data from DB");
				}
			} catch (Exception e) {
				logger.error("Error " + e.getMessage());
			}
		}

		if (formData == null) {
			MessageBox message = new MessageBox(ClientWindowManager
					.getInstance().getShell(), SWT.OK | SWT.ICON_WARNING);
			message.setText("Warning");
			message.setMessage("There was a problem retrieving the prepopulate data. Please check your Internet connection or load appointments before disconnecting.\n\n(a blank PDF will now be opened)");
			message.open();

			logger.debug("returning blank form");
			return new FlattenedForm(filePath);
		}

		logger.debug("returning new form for " + filePath);
		return new FlattenedForm(formData, filePath);
	}
}
