package za.co.momentum.recurring.nurse.util.adobe;

import java.util.Calendar;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import za.co.momentum.risk.fastlane.nurse.eforms.BaseForm;
import za.co.momentum.risk.fastlane.nurse.eforms.FormDTO;

public class FlattenedForm extends BaseForm {
  private static final long serialVersionUID = 1L;
  String filePath;

  String policyNumber;

  String clientFullName;
  String clientBirthDay;
  String clientBirthMonth;
  String clientBirthYear;
  String clientPhoneCode;
  String clientPhoneNumber;
  String clientIdNumber;
  String clientAddressLine1;
  String clientAddressLine2;
  String clientAddressPostalCode;

  String clientFirstName;
  String clientLastName;

  String nurseFullName;
  String nurseFullNameField = "XxX";

  String nurseAddressLine1;
  String nurseAddressLine2;
  String nurseAddressPostalCode;
  String nurseQualifications;
  String nursePracticeNumber;

  String nursePhoneCode;
  String nursePhoneNumber;
  String nursePhoneCodeField = "XxX";
  String nursePhoneNumberField = "XxX";

  String nurseHPCSA;
  String nurseYearQualified;

  String requirementProstate;
  String requirementLiverFunctionAST;
  String requirementLiverFunctionGGT;
  String requirementLiverFunctionALT;
  String requirementHivAndCotinine;
  String requirementCorpuscularVolume;
  String requirementHbA1C;
  String requirementBloodSugar;
  String requirementHivElisa;
  String requirementFastCholesHDLTrig;
  String requirementFastSugar;
  String requirementRandCholesterol;
  String requirementCreatinineEst;
  String requirementBloodCount;
  String requirementUrineCannabis;
  String requirementUrineCocaine;
  String requirementFastCholesHDL;
  String requirementSerumUrea;
  String requirementUrineMicroAlbumin;
  String requirementRandHdlCholesterol;
  String requirementMicroChemUrinalysis;

  public FlattenedForm(String filePath) {
    this.filePath = filePath;
  }

  public FlattenedForm(FormDTO dto, String filePath) {
    this(filePath);

    Pattern pattern = Pattern.compile(".*_(\\d{3}).*");
    Matcher matcher = pattern.matcher(filePath);
    matcher.find();
    int formNumber = Integer.parseInt(matcher.group(1));

    this.policyNumber = dto.getPolicyNumber();

    this.clientFullName = dto.getClientFirstName() + " " + dto.getClientSurname();
    this.clientFirstName = dto.getClientFirstName();
    this.clientLastName = dto.getClientSurname();

    dto.getClientDateOfBirth().setTimeZone(TimeZone.getDefault());
    this.clientBirthDay = padNumber(dto.getClientDateOfBirth().get(Calendar.DATE), 2);
    this.clientBirthMonth = padNumber(dto.getClientDateOfBirth().get(Calendar.MONTH) + 1, 2);
    this.clientBirthYear = padNumber(dto.getClientDateOfBirth().get(Calendar.YEAR), 4);

    Phone clientPhone = parsePhone(dto.getClientPhoneNumber());
    this.clientPhoneCode = clientPhone.code;
    this.clientPhoneNumber = clientPhone.number;

    this.clientIdNumber = dto.getClientIdNumber();
    if (formNumber < 21) {
      this.clientAddressLine1 = dto.getClientAddressLine1();
      this.clientAddressLine2 = dto.getClientAddressLine2();
      this.clientAddressPostalCode = dto.getClientAddressPostalCode();
    }

    if ((formNumber != 3) && (formNumber != 27) && (formNumber != 28) && (formNumber != 34)) {
      this.nurseFullName = dto.getNurseFirstName() + " " + dto.getNurseSurname();
      this.nurseFullNameField = "Medical doctors name";

      this.nurseAddressLine1 = dto.getNurseAddressLine1();
      this.nurseAddressLine2 = dto.getNurseAddressLine2();
      this.nurseAddressPostalCode = dto.getNurseAddressPostalCode();
      this.nurseQualifications = dto.getNurseQualifications();
      this.nursePracticeNumber = dto.getNursePracticeNumber();

      Phone nursePhone = parsePhone(dto.getNursePhoneNumber());
      this.nursePhoneCode = nursePhone.code;
      this.nursePhoneNumber = nursePhone.number;
      this.nursePhoneCodeField = "Dr Telephone code";
      this.nursePhoneNumberField = "Dr Telephone";
    } else if (formNumber == 3) {
      this.nurseFullName = dto.getNurseFirstName() + " " + dto.getNurseSurname();
      this.nurseFullNameField = "Name of person drawing blood sample";

      this.nursePracticeNumber = dto.getNursePracticeNumber();

      Phone nursePhone = parsePhone(dto.getNursePhoneNumber());
      this.nursePhoneCode = nursePhone.code;
      this.nursePhoneNumber = nursePhone.number;
      this.nursePhoneCodeField = "C Telephone number code";
      this.nursePhoneNumberField = "C Telephone number";
    }

    this.nurseHPCSA = dto.getNurseHPCSA();
    this.nurseYearQualified = dto.getNurseYearQualified();

    if (formNumber == 3) {
      this.requirementProstate = toYesOrNo(dto.isRequirementProstate());
      this.requirementLiverFunctionAST = toYesOrNo(dto.isRequirementLiverFunctionAST());
      this.requirementLiverFunctionGGT = toYesOrNo(dto.isRequirementLiverFunctionGGT());
      this.requirementLiverFunctionALT = toYesOrNo(dto.isRequirementLiverFunctionALT());
      this.requirementHivAndCotinine = toYesOrNo(dto.isRequirementHivAndCotinine());
      this.requirementCorpuscularVolume = toYesOrNo(dto.isRequirementCorpuscularVolume());
      this.requirementHbA1C = toYesOrNo(dto.isRequirementHbA1C());
      this.requirementBloodSugar = toYesOrNo(dto.isRequirementBloodSugar());
      this.requirementHivElisa = toYesOrNo(dto.isRequirementHivElisa());
      this.requirementFastCholesHDLTrig = toYesOrNo(dto.isRequirementFastCholesHDLTrig());
      this.requirementFastSugar = toYesOrNo(dto.isRequirementFastSugar());
      this.requirementRandCholesterol = toYesOrNo(dto.isRequirementRandCholesterol());
      this.requirementCreatinineEst = toYesOrNo(dto.isRequirementCreatinineEst());
      this.requirementBloodCount = toYesOrNo(dto.isRequirementBloodCount());
      this.requirementUrineCannabis = toYesOrNo(dto.isRequirementUrineCannabis());
      this.requirementUrineCocaine = toYesOrNo(dto.isRequirementUrineCocaine());
      this.requirementFastCholesHDL = toYesOrNo(dto.isRequirementFastCholesHDL());
      this.requirementSerumUrea = toYesOrNo(dto.isRequirementSerumUrea());
      this.requirementUrineMicroAlbumin = toYesOrNo(dto.isRequirementUrineMicroAlbumin());
      this.requirementRandHdlCholesterol = toYesOrNo(dto.isRequirementRandHdlCholesterol());
      this.requirementMicroChemUrinalysis = toYesOrNo(dto.isRequirementMicroChemUrinalysis());
    }
  }

  private static String padNumber(int value, int size) {
    return String.format("%1$0" + size + "d", value);
  }

  private static String toYesOrNo(boolean value) {
    return (value ? "Yes" : "No");
  }

  private static Phone parsePhone(String phoneString) {
    Phone result = new Phone();

    StringBuilder code = new StringBuilder();
    StringBuilder number = new StringBuilder();

    if (phoneString.startsWith("SIP")) {
      code.append("  x");
    }

    for (char c : phoneString.toCharArray()) {
      if (!Character.isDigit(c)) {
        continue;
      }

      if (code.length() < 3) {
        code.append(c);
      } else {
        number.append(c);
      }
    }

    result.code = code.toString();
    result.number = number.toString();

    return result;
  }

  private static class Phone {
    public String code;
    public String number;
  }

}
