package za.co.momentum.risk.fastlane.nurse.services.teleunderwriting;

import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.ProcessingException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import org.apache.log4j.Logger;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;

import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.ShieldUtils;
import za.co.momentum.risk.fastlane.nurse.dto.teleunderwriting.TeleUwWebServiceAppReqTO;
import za.co.momentum.risk.fastlane.nurse.exceptions.InvalidDataException;
import za.co.momentum.risk.fastlane.nurse.exceptions.MissingDataException;

public class TeleUnderwritingManagerRest implements TeleUnderwritingManager {

	private static final Logger LOG = Logger
			.getLogger(TeleUnderwritingManagerRest.class);

	private WebTarget serviceTarget;
	private static TeleUnderwritingManagerRest instance;

	public static TeleUnderwritingManagerRest getInstance() {
		if (instance == null) {
			instance = new TeleUnderwritingManagerRest();
		}
		return instance;
	}

	private TeleUnderwritingManagerRest() {
		TrustManager[] certs = new TrustManager[] { new X509TrustManager() {
			@Override
			public X509Certificate[] getAcceptedIssuers() {
				return null;
			}

			@Override
			public void checkServerTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}

			@Override
			public void checkClientTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}
		} };
		SSLContext ctx = null;
		try {
			ctx = SSLContext.getInstance("TLSv1.2");
			ctx.init(null, certs, new SecureRandom());
		} catch (java.security.GeneralSecurityException e) {

		}
		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());

		ClientBuilder clientBuilder = ClientBuilder.newBuilder();
		try {
			clientBuilder.sslContext(ctx);
			clientBuilder.hostnameVerifier(new HostnameVerifier() {

				@Override
				public boolean verify(String hostname, SSLSession session) {
					return true;
				}
			});
		} catch (Exception e) {
		}

		Client c = clientBuilder.withConfig(new ClientConfig()).build();
		String url = ClientProperties.getInstance().getProperty(
				"fastlane.teleUWService");
		serviceTarget = c.target(url);
		ShieldUtils su = new ShieldUtils();
		serviceTarget.register(HttpAuthenticationFeature.basic(
				ClientProperties.getInstance().getProperty("fastlane.user"), 
			    su.translateKey(ClientProperties.getInstance().getProperty("fastlane.password"))));
	}

	@Override
	public long createTeleUnderwritingAppReq(TeleUwWebServiceAppReqTO request)
			throws InvalidDataException, MissingDataException {
		Response response = null;
		try {
			response = serviceTarget.path("createTeleUnderwritingAppReq")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Long.class);
			} else if (response.getStatus() == 400) {
				String s = response.readEntity(String.class);
				if (s != null && s.contains("Missing")) {
					throw new MissingDataException("Request:" + request);
				} else {
					throw new InvalidDataException("Request:" + request);
				}
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error createTeleUnderwritingAppReq: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public String getFastlaneMedPersonFromRequirements(String[] request) {
		Response response = null;
		try {
			response = serviceTarget
					.path("getFastlaneMedPersonFromRequirements").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error getFastlaneMedPersonFromRequirements: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public String getFastlaneMedPersonFromRequirementsForBranchCode(
			String[] mCodes, String branchCode) {
		Response response = null;
		try {
			TeleUnderwritingRequest request = new TeleUnderwritingRequest();
			request.setRequirementMCodes(mCodes);
			request.setBranchCode(branchCode);
			response = serviceTarget
					.path("getFastlaneMedPersonFromRequirementsForBranchCode")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(String.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String
						.format("Error getFastlaneMedPersonFromRequirementsForBranchCode: %s %s",
								response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public boolean isMedPersonAvailableForSuburb(String type, String suburb) {
		Response response = null;
		try {
			TeleUnderwritingRequest request = new TeleUnderwritingRequest();
			request.setMedPersonType(type);
			request.setSuburbName(suburb);
			response = serviceTarget.path("isMedPersonAvailableForSuburb")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Boolean.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error isMedPersonAvailableForSuburb: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public long createWebNewBusinessAppReq(TeleUwWebServiceAppReqTO request)
			throws InvalidDataException, MissingDataException {
		Response response = null;
		try {

			response = serviceTarget.path("createWebNewBusinessAppReq")
					.request().post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(Long.class);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error createWebNewBusinessAppReq: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

}
