package za.co.momentum.risk.fastlane.nurse.services.user;

import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.ProcessingException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import org.apache.log4j.Logger;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;

import za.co.momentum.recurring.exceptions.DomainRuntimeException;
import za.co.momentum.recurring.nurse.util.ClientProperties;
import za.co.momentum.recurring.nurse.util.ShieldUtils;
import za.co.momentum.risk.fastlane.exceptions.DuplicateEntityException;
import za.co.momentum.risk.fastlane.exceptions.EntityNotFoundException;
import za.co.momentum.risk.fastlane.exceptions.UnauthorizedUserException;
import za.co.momentum.risk.fastlane.nurse.dto.RoleTO;
import za.co.momentum.risk.fastlane.nurse.dto.UserTO;
import za.co.momentum.risk.fastlane.nurse.exceptions.MandatoryFieldsException;

public class UserServiceRest implements UserService {

	private static final Logger LOG = Logger.getLogger(UserServiceRest.class);

	private WebTarget userTarget;

	private static UserServiceRest instance;

	public static UserServiceRest getInstance() {
		if (instance == null) {
			instance = new UserServiceRest();
		}
		return instance;
	}

	private UserServiceRest() {
		TrustManager[] certs = new TrustManager[] { new X509TrustManager() {
			@Override
			public X509Certificate[] getAcceptedIssuers() {
				return null;
			}

			@Override
			public void checkServerTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}

			@Override
			public void checkClientTrusted(X509Certificate[] chain,
					String authType) throws CertificateException {
			}
		} };
		SSLContext ctx = null;
		try {
			ctx = SSLContext.getInstance("TLSv1.2");
			ctx.init(null, certs, new SecureRandom());
		} catch (java.security.GeneralSecurityException e) {

		}
		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());

		ClientBuilder clientBuilder = ClientBuilder.newBuilder();
		try {
			clientBuilder.sslContext(ctx);
			clientBuilder.hostnameVerifier(new HostnameVerifier() {

				@Override
				public boolean verify(String hostname, SSLSession session) {
					return true;
				}
			});
		} catch (Exception e) {
		}

		Client c = clientBuilder.withConfig(new ClientConfig()).build();
		String url = ClientProperties.getInstance().getProperty(
				"fastlane.userService");
		userTarget = c.target(url);
		ShieldUtils su = new ShieldUtils();
		userTarget.register(HttpAuthenticationFeature.basic(
				ClientProperties.getInstance().getProperty("fastlane.user"), 
			    su.translateKey(ClientProperties.getInstance().getProperty("fastlane.password"))));
	}

	@Override
	public UserTO getUserByName(String username) throws EntityNotFoundException {
		try {
			Response response = userTarget.path("/getUserByName/" + username)
					.request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(UserTO.class);
			} else if (response.getStatusInfo() == Response.Status.NOT_FOUND) {
				throw new EntityNotFoundException(username);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getUserByName error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getUserByName:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public UserTO getUser(long id) throws EntityNotFoundException {
		try {
			Response response = userTarget.path("/getUser/" + id).request()
					.get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(UserTO.class);
			} else if (response.getStatusInfo() == Response.Status.NOT_FOUND) {
				throw new EntityNotFoundException("userid:" + id);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getUser error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getUser:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public UserTO createUser(UserTO userTO) throws DuplicateEntityException,
			MandatoryFieldsException, EntityNotFoundException {
		Response response = null;
		try {
			response = userTarget.path("createUser").request()
					.post(Entity.json(userTO));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(UserTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("User:" + userTO);
			} else if (response.getStatus() == 409) {
				throw new DuplicateEntityException("User:" + userTO);
			} else if (response.getStatus() == 406) {
				throw new MandatoryFieldsException("User:" + userTO);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error createUser: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}

	}

	@Override
	public UserTO updateUser(UserTO userTO) throws DuplicateEntityException,
			MandatoryFieldsException, EntityNotFoundException {
		Response response = null;
		try {
			response = userTarget.path("updateUser").request()
					.post(Entity.json(userTO));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(UserTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("User:" + userTO);
			} else if (response.getStatus() == 409) {
				throw new DuplicateEntityException("User:" + userTO);
			} else if (response.getStatus() == 406) {
				throw new MandatoryFieldsException("User:" + userTO);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error updateUser: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void checkDuplicateUsername(String username)
			throws DuplicateEntityException, MandatoryFieldsException {
		Response response = null;
		try {
			response = userTarget.path("checkDuplicateUsername").request()
					.post(Entity.json(username));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else if (response.getStatus() == 409) {
				throw new DuplicateEntityException("User:" + username);
			} else if (response.getStatus() == 406) {
				throw new MandatoryFieldsException("User:" + username);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format(
						"Error checkDuplicateUsername: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}

	}

	@Override
	public UserTO[] findUsers(String firstName, String surname)
			throws MandatoryFieldsException, EntityNotFoundException {
		Response response = null;
		try {
			FindUsersRequest request = new FindUsersRequest();
			request.setFirstName(firstName);
			request.setSurname(surname);
			response = userTarget.path("findUsers").request()
					.post(Entity.json(request));

			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(UserTO[].class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("User:" + firstName + " "
						+ surname);
			} else if (response.getStatus() == 406) {
				throw new MandatoryFieldsException("User:" + firstName + " "
						+ surname);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error findUsers: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void removeNurseRole(long id) throws EntityNotFoundException {
		Response response = null;
		try {
			response = userTarget.path("removeNurseRole").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("User:" + id);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error removeNurseRole: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public void removeSysAdminRole(long id) throws EntityNotFoundException {
		Response response = null;
		try {
			response = userTarget.path("removeSysAdminRole").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("User:" + id);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error removeSysAdminRole: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}

	}

	@Override
	public void removeDcRole(long id) throws EntityNotFoundException {
		Response response = null;
		try {
			response = userTarget.path("removeDcRole").request()
					.post(Entity.json(id));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				response.readEntity(String.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("User:" + id);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error removeDcRole: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public UserTO logon(String username, String password)
			throws UnauthorizedUserException, EntityNotFoundException {
		Response response = null;
		try {
			LogonRequest request = new LogonRequest();
			request.setUsername(username);
			request.setPassword(password);
			response = userTarget.path("logon").request()
					.post(Entity.json(request));
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(UserTO.class);
			} else if (response.getStatus() == 404) {
				throw new EntityNotFoundException("User:" + username);
			} else if (response.getStatus() == 401) {
				throw new UnauthorizedUserException("User:" + username);
			} else {
				String s = response.readEntity(String.class);
				String error = String.format("Error logon: %s %s",
						response.getStatus(), s);
				LOG.error(error);
				throw new DomainRuntimeException(error);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		} finally {
			if (response != null) {
				response.close();
			}
		}
	}

	@Override
	public RoleTO[] getAllNurses() {
		try {
			Response response = userTarget.path("/getAllNurses").request()
					.get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(RoleTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getAllNurses error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getAllNurses:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public RoleTO[] getAllThirdPartyNurses() {
		try {
			Response response = userTarget.path("/getAllThirdPartyNurses")
					.request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(RoleTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getAllThirdPartyNurses error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException(
						"Error getAllThirdPartyNurses:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public RoleTO[] getAllDcs() {
		try {
			Response response = userTarget.path("/getAllDcs").request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(RoleTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getAllDcs error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getAllDcs:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public RoleTO[] getNursesBySuburb(String suburb)
			throws EntityNotFoundException {
		try {
			Response response = userTarget.path("/getNursesBySuburb/" + suburb)
					.request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(RoleTO[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getNursesBySuburb error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getNursesBySuburb:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public String[] getThirdPartyCompanies() {
		try {
			Response response = userTarget.path("/getThirdPartyCompanies")
					.request().get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(String[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getThirdPartyCompanies error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException(
						"Error getThirdPartyCompanies:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

	@Override
	public String[] getContractNurses() {
		try {
			Response response = userTarget.path("/getContractNurses").request()
					.get();
			response.bufferEntity();
			if (response.getStatusInfo().getFamily() == Response.Status.Family.SUCCESSFUL) {
				return response.readEntity(String[].class);
			} else {
				String s = response.readEntity(String.class);
				LOG.error("Get getContractNurses error. ResponseCode: "
						+ response.getStatus() + s);
				throw new DomainRuntimeException("Error getContractNurses:" + s);
			}
		} catch (ProcessingException e) {
			LOG.error(e.getMessage());
			throw e;
		}
	}

}
